# coding: utf-8

import Tkinter
from gui import convweak

class NamePlate(Tkinter.Frame):
    def __init__(self,master=None,name='',cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        Tkinter.Label(self,text=name).pack(side=Tkinter.TOP,expand=1,fill=Tkinter.X)

class WeakValuedButton(Tkinter.Button):
    '''クリックすると登録したコールバックを起動する。コールバックは通常のTkinter.Buttonと同様にcommand=<method>オプションで指定できる。'''
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.configure(command=self._clicked) # ボタンへのコールバックを設定する
    def _configure(self,cmd,cnf,kw):
        command = None
        if cnf:
            if cnf.has_key('command'):
                command = cnf['command']
                cnf['command'] = self._clicked
        if kw: # もしkwが存在して
            if kw.has_key('command'): # 'command'キーを持っていれば
                command = kw['command'] # コマンドの内容を控え
                kw['command'] = self._clicked # 弱参照呼び出しコールバックにすり替える
        if command: # commandが設定されていれば
            self._callback = convweak(command) # callbackにメソッドの弱参照を設定し
        Tkinter.Button._configure(self,cmd,cnf,kw) # 設定コマンドを呼び出す
    def _clicked(self):
        try: self._callback # もしコールバックが定義されていて
        except AttributeError: pass
        else:
            if not self._callback[0]() == None: # 参照先が削除されていなければ
                wobj, method = self._callback
                getattr(wobj(),method).__call__() # メソッドをコールする
            else:
                del self._callback # 参照先が存在していなければコールバックを削除する



# coding: utf-8

import weakref, types, inspect, platform, types, numpy
import Tkinter
import ttk #@UnresolvedImport
import core, widgets, utils
from functools import partial

def limit_depth(map,depth=None):
    if not depth == None:
        remove = []
        for key in map:
            if len(key.split('.')) > depth:
                remove.append(key)
        for key in remove:
            del map[key]
    return map

def pathtoid(path):
    if len(path) == 0: return ''
    id = path[0]
    for s in path[1:]:
        id = '%s.%s' % (id,s)
    return id

def classname(obj): return repr(obj.__class__).split('.')[-1].split(' ')[0]

def widget(object,id):
    path   = id.split('.')
    def mycore(parent):
        children = parent.children
        for key in children:
            child = parent.children[key]
            if utils.isfamily(child,BasePanel): continue # パネルに到達したら別の枝を探索
            if not key == path[0]: # 名前が一致しなければ
                target = mycore(child) # さらに探索する
                if target == None:
                    continue
                else:
                    return target
            ports = inspect.getmembers(child,core.isPort)
            if not len(ports) == 0: # 名前が一致してかつPyLAFウィジェットならば
                path.pop(0) # 探索パスを１段深くして
                if len(path) == 0: return child # 最終段であれば child が対象
                target = mycore(child)
                if target == None:
                    continue
                else:
                    return target
    return mycore(object)

def console(self,path=None):
    '''
    配下のコンソールを取得する（コンソール以外のウィジェットは無視する）
    ワイルドカード * が与えられた場合は、最初に見つかったコンソールを返す
    pathが与えられなかった場合は、最初に見つかったコンソールを返す
    もしコンソールが見つからなければ None を返す
    '''
    if path == None:
        path = ['*']
    else:
        path = path.split('.')
    def mycore(parent):
        for key in parent.children:
            child = parent.children[key]
            if not utils.isfamily(child,BaseConsole): # もしconsoleでなければ
                target = mycore(child) # さらに探索し
                if target == None:
                    continue # 見つからなければ別のブランチを探索する
                else:
                    return target # 見つかればtargetを返す
            if path[0] == '*': # ワイルドカードならば
                return child # childを返す
            if key == path[0]: # もし名前が一致すれば
                path.pop(0) # 探索パスを１段深くして
                if len(path) == 0: return child # 最終段であれば child が対象
                target = mycore(child) # 最終段でなければさらに探索を続ける
                if target == None:
                    continue
                else:
                    return target
            # 一致しなければ探索を打ち切り別のブランチを検索する
    return mycore(self)
    
class Layout:
    '''
    Panelの配置とコンポーネント割付を管理する
    '''
    def available_panels(self):
        '''
        直下のすべてのパネルを格納した辞書を返す
        '''
        panels       = {}
        current_path = []
        def core(parent):
            for key in parent.children:
                child = parent.children[key]
                if utils.isfamily(child, BaseConsole): # Consoleを見つけたら探索を打ち切る
                    continue
                if utils.isfamily(child, BasePanel): # Panelを見つけたらパスの末尾にキーを追加したうえで辞書に追加
                    current_path.append(key)
                    panels[pathtoid(current_path)] = child
                core(child)
                if utils.isfamily(child, BasePanel): # Panelから離脱する場合はパスの末尾を削除する
                    current_path.pop(-1)
        core(self)
        return panels
    def panel(self,id):
        panels = self.available_panels()
        if panels.has_key(id):
            return panels[id]
        else:
            return
    # TODO: core.PortでSubjectへ同一ポートやコールバックを複数回登録しないように変更すること
    # TODO: linkmapをどこで設定するのか．．．？
    # TODO: assignmapをどこで設定するべきか検討する
    def component(self,path=None):
        return self.comp().component(path)
    def assign(self,component):
        self.comp = weakref.ref(component)
        available_panels = limit_depth(self.available_panels(),depth=1)
        for key in available_panels: # デフォルトではカレントコンポーネントを割り振る
            self.panel(key).assign(component)
    def popup_config(self,master,**kw):
        component = self.component()
        try: component.Config
        except AttributeError: return
        notebook = ttk.Notebook(master,name='config',**kw)
        notebook.pack(expand=True,fill=Tkinter.X)
        panels = inspect.getmembers(component.Config,inspect.isclass)
        subpanels = []
        for name,klass in panels:
            panel = klass(notebook,name=name.lower())
            subpanels.append(panel)
            notebook.add(panel,text=name)
        return subpanels
        
class Standard(Tkinter.Frame,Layout):
    def __init__(self,master=None,klass=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        try: klass.Plot
        except AttributeError: pass
        else:
            klass.Plot(self,name='plot').grid(row=1,column=0,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
            self.grid_rowconfigure(1,weight=1)
        try: klass.Control
        except AttributeError:
            control = ControlPanelBuilder(self,klass,name='control')
        else:
            control = klass.Control(self,name='control')
        control.grid(row=2,column=0,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
        #self.grid_rowconfigure(2,weight=1)
        try: klass.Status
        except AttributeError: pass
        else:
            klass.Status(self,name='status').grid(row=3,column=0,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
            #self.grid_rowconfigure(3,weight=1)
        try: klass.Toolbar
        except AttributeError: pass
        else:
            klass.Toolbar(self,name='toolbar').grid(row=0,column=0,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
            #self.grid_rowconfigure(0,weight=1)
        self.grid_columnconfigure(0,weight=1)
        
class StandardH(Tkinter.Frame,Layout):
    def __init__(self,master=None,klass=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        try: klass.Plot
        except AttributeError: pass
        else:
            klass.Plot(self,name='plot').grid(row=1,column=0,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
            self.grid_rowconfigure(1,weight=1)
        try: klass.Control
        except AttributeError:
            control = ControlPanelBuilder(self,klass,name='control')
        else:
            control = klass.Control(self,name='control')
        control.grid(row=1,column=1,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
        #self.grid_rowconfigure(1,weight=1)
        try: klass.Status
        except AttributeError: pass
        else:
            klass.Status(self,name='status').grid(row=2,column=0,columnspan=2,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
            #self.grid_rowconfigure(2,weight=1)
        try: klass.Toolbar
        except AttributeError: pass
        else:
            klass.Toolbar(self,name='toolbar').grid(row=0,column=0,columnspan=2,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
            #self.grid_rowconfigure(0,weight=1)
        self.grid_columnconfigure(0,weight=1)
        
class PlotLayout(Tkinter.Frame,Layout):
    def __init__(self,master=None,klass=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        klass.Plot(self,name='plot').pack(expand=True,fill=Tkinter.BOTH)
    def popup_config(self,master,**kw):
        component = self.component()
        try: component.Config
        except AttributeError:
            try: component.Control
            except AttributeError: return
        notebook = ttk.Notebook(master,name='config',**kw)
        notebook.pack(expand=True,fill=Tkinter.X)
        subpanels = []
        try: component.Config
        except AttributeError: pass
        else:
            panels = inspect.getmembers(component.Config,inspect.isclass)
            for name,klass in panels:
                panel = klass(notebook,name=name.lower())
                subpanels.append(panel)
                notebook.add(panel,text=name)
        try: component.Control
        except AttributeError:
            panel = ControlPanelBuilder(self,component.__class__,name='control')
        else:
            panel = component.Control(notebook,name='control')
        subpanels.append(panel)
        notebook.add(panel,text='Control')
        return subpanels
        
class ControlLayout(Tkinter.Frame,Layout):
    def __init__(self,master=None,klass=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        try: klass.Control
        except AttributeError:
            control = ControlPanelBuilder(self,klass,name='control')
        else:
            control = klass.Control(self,name='control')
        control.pack(expand=True,fill=Tkinter.X)
        
class TableBuilder:
    def __init__(self,master,column=2):
        self.master = weakref.ref(master)
        self.column = column
        col, row    = master.grid_size()
        self.cursor = (0, row)
    def add(self,klass,*args,**kwargs):
        self._item = weakref.ref(klass(self.master(),*args,**kwargs))
        return self
    def grid(self,row=None,column=None,rowspan=None,columnspan=1,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S,**kwargs):
        # rowspan は無視
        column, row = self.cursor
        self._item().grid(row=row,column=column,columnspan=columnspan,sticky=sticky,**kwargs)
        #self.master().grid_rowconfigure(row,weight=1)
        self.master().grid_columnconfigure(column,weight=1)
        if (column + columnspan) <= (self.column - 1):
            column = column + columnspan
        else:
            column = 0
            row = row + 1
        self.cursor = (column, row)
        return self
    def item(self):
        return self._item()
    def bind(self,*args,**kwargs):
        return self.item().bind(*args,**kwargs)
        
class BasePanel:
    '''
    パネルの生成
        ウィジェットの配置
        コンソールの配置
    コンポーネントをアサインする
    '''
    def available_panels(self):
        '''
        直下のすべてのパネルを格納した辞書を返す
        '''
        panels       = {}
        current_path = []
        def core(parent):
            for key in parent.children:
                child = parent.children[key]
                if utils.isfamily(child, BaseConsole): # Consoleを見つけたら探索を打ち切る
                    continue
                if utils.isfamily(child, BasePanel): # Panelを見つけたらパスの末尾にキーを追加したうえで辞書に追加
                    current_path.append(key)
                    panels[pathtoid(current_path)] = child
                core(child)
                if utils.isfamily(child, BasePanel): # Panelから離脱する場合はパスの末尾を削除する
                    current_path.pop(-1)
        core(self)
        return panels
    def available_consoles(self):
        '''
        配下のすべてのコンソールを格納した辞書を返す
        '''
        consoles     = {}
        current_path = []
        def core(parent):
            for key in parent.children:
                child = parent.children[key]
                if utils.isfamily(child, BaseConsole):
                    current_path.append(key)
                    consoles[pathtoid(current_path)] = child
                    current_path.pop(-1)
                else:
                    core(child)
        core(self)
        return consoles
    def port(self,id):
        path = id.split('.')
        path, name = path[:-1], path[-1]
        def myport(object,name):
            try: port = getattr(object,name)
            except AttributeError: # メンバが存在しなくて
                #children = object.children
                #print name, children
                child = widget(object,name)
                #if not children.has_key(name): return # 子ウィジェットがなければ None
                #child = children[name]
                if child == None: return # 子ウィジェットがなければ None
                try: default = getattr(child,'sig_in')
                except AttributeError: pass
                try: default = getattr(child,'sigin')
                except AttributeError: pass
                try: default = getattr(child,'value')
                except AttributeError: pass
                try: default # デフォルトポートがあるか調べて
                except AttributeError: return # 存在しなければ None
                port = default # ポートをデフォルトポートに設定する
            if not core.isPort(port): return # ポートオブジェクトでなければ None
            return port
        def mycore(parent): # descent path
            if len(path) == 0: return myport(parent,name)
            children = parent.children
            child_name = path[0]
            if not children.has_key(child_name): return # オブジェクトが存在しなければ None
            path.pop(0)
            mycore(children[child_name])
        return mycore(self)
    def available_ports(self):
        '''
        自身の配下で到達可能なすべてのポートマップを探索して返す
        同一のパス名で到達可能なポートが複数ある場合、以下の優先順位で上位のポートをマップする
        優先順位：
            [widget_name...]portname
            [widget_name...]widget_name（暗黙にプライマリーポートを指定）
            プライマリーポートの優先順位
                'value', 'sigin', 'sig_in'
        '''
        #map = {}
        map = weakref.WeakValueDictionary()
        # ウィジェットの保有するポートを探索する
        available_widgets = self.available_widgets()
        for available_widget_path in available_widgets:
            #available_widget = self.widget(available_widget_path) # ボトルネックだった
            available_widget = available_widgets[available_widget_path] # 高速化のため
            #members = inspect.getmembers(available_widget,lambda x:utils.functions.isfamily(x,core.Port))
            members = inspect.getmembers(available_widget,core.isPort)
            prime = {}
            for name,port in members:
                map['%s.%s' % (available_widget_path,name)] = port
                if   name == 'value':  prime[name] = port
                elif name == 'sigin':  prime[name] = port
                elif name == 'sig-in': prime[name] = port
            if   prime.has_key('value'):  map[available_widget_path] = prime['value']
            elif prime.has_key('sigin'):  map[available_widget_path] = prime['sigin']
            elif prime.has_key('sig-in'): map[available_widget_path] = prime['sig-in']
        # パネルの保有するポートを探索する
        #members = inspect.getmembers(self,lambda x:utils.functions.isfamily(x,core.Port))
        members = inspect.getmembers(self,core.isPort)
        for name,port in members:
            map[name] = port
        return map
    #
    # 配下のウィジェットからポートを保有するものだけを選んで論理パスをキーとした辞書を返す
    def available_widgets(self):
        widgets = {}
        currentpath = []
        def pathtoid(path):
            if len(path) == 0: return ''
            id = path[0]
            for s in path[1:]:
                id = '%s.%s' % (id,s)
            return id
        def mycore(parent):
            for key in parent.children:
                child = parent.children[key]
                if utils.isfamily(child,BasePanel):
                    continue
                #ports = inspect.getmembers(child,lambda x:utils.isfamily(x,core.Port))
                ports = inspect.getmembers(child,core.isPort)
                if not len(ports) == 0:
                    currentpath.append(key)
                    widgets[pathtoid(currentpath)] = child
                mycore(child)
                if not len(ports) == 0:
                    currentpath.pop(-1)
        mycore(self)
        return widgets
    console = console
    #def console(self,id):
    #    consoles = self.available_consoles()
    #    if consoles.has_key(id):
    #        return consoles[id]
    #    else:
    #        return
    def clear_portcache(self):
        try: self.ports
        except AttributeError: return
        del self.ports
    def port_old(self,id):
        '''
        自身の配下のpathで指定されるポートを返す
        注意！：最初に呼ばれたときにキャッシュを生成するので
              ウィジェット構造が確定してから利用すること
        　　　　ウィジェット構造が変化した場合はclear_portcacheを実行してください
        path:
            書式１：[widget_name...]portname
            書式２：[widget_name...]widget_name（暗黙にプライマリーポートを指定）
            到達可能な複数のポートが存在する場合は書式１、２の順で優先適用する
        もし存在しなければNoneを返す
        '''
        try: self.ports
        except AttributeError:
            self.ports = self.available_ports()
        if self.ports.has_key(id):
            return self.ports[id]
        else:
            return
    def widget(self,id):
        widgets = self.available_widgets()
        if widgets.has_key(id):
            return widgets[id]
        else:
            return
    def assign_panel(self,component):
        '''
        直近の配下パネルへのアサイン
        '''
        available_panels = limit_depth(self.available_panels(),depth=1)
        for key in available_panels:
            available_panels[key].assign(component)
    def assign_console(self,component):
        '''
        直近の配下コンソールへのアサイン
        '''
        # map: 対応関係を格納する辞書 'コンソールパス' : 'コンポーネントパス'
        map = {}
        # 優先順位２：同一パスのコンポーネント
        for console_name in limit_depth(self.available_consoles(),depth=1):
            map[console_name] = console_name
        # 優先順位１：assignmap
        try: self.assignmap
        except AttributeError: pass
        else:
            for console_path in self.assignmap:
                component_path = self.assignmap[console_path]
                if not len(console_path.split('.')) == 1:
                    map[console_path] = component_path
        # mapに従ってcomponentをconsoleへassignする
        for console_path in map:
            component_path = map[console_path]
            child          = component.child(component_path)
            console        = self.console(console_path)
            if child == None: # もし対象となるコンポーネントが存在しなければ匿名で生成する
                component_name = component_path.split('.')[-1]
                child = console.klass(component)# ,name=component_name)
                console.sync_on()
                # 親コンポーネントのsigoutとsiginをバインドする
                try: component.sig_out
                except AttributeError: pass
                else: src = component.sig_out
                try: component.sigout
                except AttributeError: pass
                else: src = component.sigout
                try: src
                except NameError: continue
                #
                try: child.sig_in
                except AttributeError: pass
                else: dst = child.sig_in
                try: child.sigin
                except AttributeError: pass
                else: dst = child.sigin
                try: dst
                except NameError: continue
                src.link(dst)
            console.assign(child)
    def assign(self,component):
        '''
        コンポーネントをアサインする
        componentは事前に存在していなければならない
        '''
        self.assign_panel(component)   # 直近の配下パネルへのアサイン
        self.assign_console(component) # 直近の配下コンソールへのアサイン
        '''
        ポートをリンクする
        '''
        # map: 対応関係を格納する辞書 'ターゲットポートパス' : 'ソースポートパス'
        map = {}
        # 優先順位２：同一パスのコンポーネントのポート
        for target_path in self.available_ports(): # 配下のすべてのポートが対象
            # Component 以外が与えられた場合、単純な参照を行う（ControlPanelBuilderのために）
            try: component.port
            except AttributeError:
                try: getattr(component,target_path)
                except AttributeError: pass
                else: map[target_path] = target_path
            else:
                if not component.port(target_path) == None: # リンク対象が存在していれば
                    map[target_path] = target_path
        # 優先順位１：linkmap
        try: self.linkmap
        except AttributeError: pass
        else:
            for target_path in self.linkmap:
                source_path = self.linkmap[target_path]
                if source_path == None:
                    del map[target_path] # source_path が None ならリンク取り消し
                else:
                    map[target_path] = source_path
                #component.port(source_path).link(self.port(target_path),set=True)
        #     配下のポートを保有するウィジェット（パネルとコンソールを除く）すべてを対象にリンクする
        #         1) linkmap
        #             getattr(component.child(path),name).link(getattr(self.widget(path),name),set=True)
        #         2) 同一名のコンポーネントのポート
        #             getattr(component,name).link(getattr(self.widget(path),name),set=True)
        #     自身の保有するすべてのポートを対象にリンクする
        #         1) linkmap
        #             getattr(component.child(path),name).link(getattr(self,name),set=True)
        #         2) 同一名のコンポーネントのポート
        #             getattr(component,name).link(getattr(self,name),set=True)
        # print component, self.__class__, map
        # mapに従ってportをcomponentのportへlinkする
        for target_path in map:
            source_path = map[target_path]
            self.port(target_path).disable(self)
            # Component 以外が与えられた場合、単純参照をおこなう
            try: component.port
            except AttributeError:
                getattr(component,source_path).link(self.port(target_path),set=True)
            else:
                component.port(source_path).link(self.port(target_path),set=True)
            #
            self.port(target_path).enable()
            # 複数のポートを対象にするならば次のコードだが、setのところでパフォーマンス上の問題が予期されるので不採用
            #source_paths = map[target_path]
            #if type(source_paths) == str:
            #    source_paths = (source_paths)
            #for source_path in source_paths:
            #    component.port(source_path).link(self.port(target_path),set=True)
    def sibling(self,path):
        ''' 兄弟パネルを取得する '''
        return self.layout().panel(path)
    def layout(self):
        ''' 直近の上位レイアウトを取得する '''
        master = self.master
        while not utils.isfamily(master,Layout):
            master = master.master
        return master
    def component(self,path=None):
        return self.layout().component(path)
        
class Panel(Tkinter.Frame,BasePanel): pass

class ControlPanelBuilder(Panel):
    def __init__(self,master=None,klass=None,cnf={},**kw):
        Panel.__init__(self,master,cnf,**kw)
        # klassにClassType以外が与えられた場合、インスタンスが与えられたものとして処理する
        #if type(klass) == types.ClassType:
        #    instance = klass()
        #    destroy  = True
        #else:
        #    instance = klass
        #    klass    = klass.__class__
        #    destroy  = False
        #
        instance = klass()
        builder = TableBuilder(self)
        members = inspect.getmembers(instance,core.isPort)
        for key,port in members:
            value = port.subject.value
            if (
                type(value) == types.NoneType or
                type(value) == types.InstanceType or
                type(value) == numpy.ndarray
                ): continue
            builder.add(Tkinter.Label,text=key).grid()
            builder.add(widgets.Entry,name=key).grid()
            #self.widget(key).bind('<Return>',self._trig)
        # もし、klassがClassTypeだった場合、生成したインスタンスを破棄する
        #if destroy: instance.destroy()
        #
        try: instance.destroy()
        except AttributeError: pass
    #def _trig(self,*args,**kwargs):
    #    component = self.layout().component()
    #    members = [key for key, port in inspect.getmembers(component,core.isPort)]
    #    if   members.count('sigout'):  component.sigout.get()
    #    elif members.count('sig_out'): component.sig_out.get()
        
class Menu(Tkinter.Menu):
    TYPE_CASCADE, TYPE_ITEM = range(2)
    ITEMS = []
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Menu.__init__(self,master,cnf,**kw)
        self.callback = []
        self.make()
    def make(self,items=None):
        if not items: items = self.__class__.ITEMS
        for child in items:
            self.add_child(self,child)
    def remove_empty_items(self,cascade):
        length = cascade.len()
        for index in range(length):
            if cascade.type(index) == 'cascade':
                label = cascade.entrycget(index,'label')
                menu  = cascade.nametowidget(cascade.entrycget(index,'menu'))
                self.remove_empty_items(menu)
                if utils.isfamily(menu,ChildrenMenu):
                    continue
                if menu.len() == 0:
                    cascade.delete(index)
    def itemtype(self,item):
        cls = self.__class__
        if len(item) == 1: return cls.TYPE_CASCADE
        if type(item[1]) == list: return cls.TYPE_CASCADE
        if repr(type(item[1])) == '<type \'classobj\'>': return cls.TYPE_CASCADE
        if type(item[1]) == dict: return cls.TYPE_ITEM
    def parse_cascade(self,item):
        if len(item) == 1: return item[0], None
        if type(item[1]) == list: return item[0], item[1:]
        if repr(type(item[1])) == '<type \'classobj\'>': return item[0], item[1]
    def add_child(self,parent,child):
        def nocommand(command=None,**kw): return kw
        cls = self.__class__
        itemtype = self.itemtype(child) # child がどのような要素か調べる
        if itemtype == cls.TYPE_CASCADE: # child が カスケード であるならば
            label, item = self.parse_cascade(child) # label と item を抽出して
            index = self.labeltoindex(label)
            if index == None: # label が 不在 ならば
                if not repr(type(item)) == '<type \'classobj\'>':
                    cascade = Menu(parent,name=label.lower()) # あたらしいカスケード要素を生成して
                    parent.add_cascade(label=label,menu=cascade)
                    parent = cascade # 生成したカスケードを parent とする
            else:
                old = self.item(label)
                if old == None: # old が カスケード でなければ
                    self.delete(index) # 元の要素を削除して
                    if not repr(type(item)) == '<type \'classobj\'>':
                        cascade = Menu(parent,name=label.lower()) # 新しいカスケード要素で上書き
                        parent.add_cascade(label=label,menu=cascade)
                        parent = cascade
                    else:
                        pass
                else:
                    parent = old
            if   item == None:
                pass
            elif repr(type(item)) == '<type \'classobj\'>':
                parent.add_cascade(label=label,menu=item(parent,name=label.lower()))
            elif type(item) == list:
                for o in item:
                    self.add_child(parent,o)
        elif itemtype == cls.TYPE_ITEM:
            name, kwargs = child
            if kwargs.has_key('command'): # もしコマンドが定義されていて
                if type(kwargs['command']) == str: # オペランドがメソッド名ならば
                    command = getattr(self,kwargs['command']) # 自身のメソッドをコマンドに割り当てる
                    getattr(parent,name)(command=command,**nocommand(**kwargs))
                    return
            getattr(parent,name)(**kwargs)
    def assign(self,component):
        self.comp = weakref.ref(component)
        for key in self.children:
            child = self.children[key]
            if utils.functions.isfamily(child,Menu):
                child.assign(component)
    def component(self,path=None):
        if not path: return self.comp()
        ids = path.split('.')
        child = self.comp()
        for id in ids:
            child = child.children[id]
        return child
    def labeltoindex(self,label):
        index = 0
        while index == self.index(index):
            try: self.entrycget(index,'label')
            except: pass
            else:
                if label == self.entrycget(index,'label'):
                    return index
            index = index + 1
        return
    def len(self):
        index = 0
        while index == self.index(index):
            index = index + 1
        return index
    def item(self,label):
        index = self.labeltoindex(label)
        if not index: return
        return self.nametowidget(self.entrycget(index,'menu'))

class ChildrenMenu(Menu):
    def assign(self,component):
        Menu.assign(self,component)
        self.update_children()
    def update_children(self):
        try: self.comp # もしコンポーネントが割り付けられていなければなにもしない
        except AttributeError: return
        component = self.component()
        if component == None: # もしコンポーネントが廃棄されていれば割り付けを解除する
            del self.comp
            return
        # Component 以外が与えられた場合の処理
        try: component.children
        except AttributeError: return
        #
        children = component.children.copy()
        # メニューに存在する子Ｃが実在するかチェックしてメニューを削除する
        existence = ['%s:%s' % (key,classname(children[key])) for key in children]
        remove = []
        for index in range(self.len()):
            try:
                label = self.entrycget(index,'label')
            except: continue
            flag = False
            for s in existence:
                if s == label:
                    flag = True
                    break
            if not flag:
                remove.append(index)
        for index in remove:
            self.delete(index)
        # ツリーに存在する子Ｃがメニューアイテムに存在するかチェックしてメニューアイテムを追加する
        for s in existence:
            flag = False
            for index in range(self.len()):
                try:
                    label = self.entrycget(index,'label')
                except: continue
                if s == label:
                    flag = True
                    break
            if flag: continue
            self.add_command(label=s,command=partial(self.popup,label=s))
        self.after(500,self.update_children)
    def popup(self,label):
        key = label.split(':')[0]
        object = self.component(key)
        #
        tk = self
        while tk: tk = tk.master
        Console(Tkinter.Toplevel(tk),object,sync=False)
        
class FileMenu(Menu):
    ITEMS = []
    RESTS = [
             ['add_command', utils.kw(label='Config',command='popup_config')],
             ]
    def make_rest(self):
        cls = self.__class__
        if not self.len() == 0:
            self.add_separator()
        self.make(cls.RESTS)
    def popup_config(self):
        if self.children.has_key('config'): return # すでにconfigを生成していなければ
        component = self.component()
        console = self
        while not utils.isfamily(console,BaseConsole):
            console = console.master
        layout   = console.layout()
        toplevel = Tkinter.Toplevel(layout,name='config')
        panels = layout.popup_config(toplevel)
        if panels == None:
            toplevel.destroy()
            return
        for panel in panels:
            panel.assign(component)
            
class DefaultMenu(Menu):
    DEFAULT_ITEMS = [
                     ['PyLAF', ],
                     ['File', FileMenu],
                     ['Edit'],
                     ['Children', ChildrenMenu],
                     ]
    ITEMS = []
    #RESTS = [
    #         ['Window', ],
    #         ['Help', ],
    #         ]
    # TODO:GUI毎に管理しなければならないパラメータの取り扱い。Componentからは初期値のみ管理。
    def __init__(self,master=None,desc=[],cnf={},**kw):
        cls = self.__class__
        #ITEMS = cls.ITEMS
        #cls.ITEMS = []
        Menu.__init__(self,master,cnf,**kw)
        self.make(cls.DEFAULT_ITEMS)
        self.make(cls.ITEMS)
        self.make_rest(self.children)
        self.remove_empty_items(self)
    def make_rest(self,children): # メニューの末尾にアイテムを追加する
        #self.make(self.__class__.RESTS)
        for key in children:
            child = children[key]
            if utils.isfamily(child,Menu):
                try: make_rest = getattr(child,'make_rest')
                except AttributeError: continue
                make_rest()
                self.make_rest(child.children)

class BaseConsole:
    # panels   = panels
    # panel    = panel
    # consoles = consoles
    console  = console
    def assign(self,component):
        self.comp = weakref.ref(component)
        self.layout().assign(component)
        self.menu().assign(component)
    def component(self,path=None):
        try: self.comp
        except AttributeError: return
        if not path: return self.comp()
        ids = path.split('.')
        child = self.comp()
        for id in ids:
            child = child.children[id]
        return child
    def menu(self):
        return self.children['menu']
    def layout(self):
        return self.children['layout']
    def panel(self,id):
        return self.layout().panel(id)
        
class Console(Tkinter.Frame,BaseConsole):
    def __init__(self,master=None,component=None,layout=Standard,sync=True,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.sync = sync
        # componentがClassTypeならば雛形生成モードにする
        if type(component) == types.ClassType:
            klass = component
            component = None
        else:
            klass = component.__class__
        # レイアウトを生成する
        #self.layout = weakref.ref(layout(self,klass=klass,name='layout'))
        layout(self,klass=klass,name='layout')
        # メニューを生成する
        try: menu = klass.Menu
        except AttributeError:
            #self.menu = weakref.ref(DefaultMenu(master=self,name='menu'))
            menu = DefaultMenu
            #self.menu = weakref.ref(klass.Menu(master=self,name='menu'))
        self.master.config(menu=menu(master=self,name='menu'))
        #self.master.config(menu=self.menu())
        # componentをアサインする
        if not component == None:
            self.assign(component)
        self.layout().pack(expand=True,fill=Tkinter.BOTH)
        self.pack(expand=True,fill=Tkinter.BOTH)
        if sync:
            self.polling()
    def polling(self):
        try: self.comp # すでにassignされていて
        except AttributeError: pass
        else:
            if self.component() == None: # componentが死んでいたら
                self.destroy() # 自爆して
                return # ループを抜ける
        self.after(500,self.polling)
    def destroy(self):
        if self.sync and (not self.component() == None):
            self.component().destroy()
        Tkinter.Frame.destroy(self)
        
class Embed(Tkinter.LabelFrame,BaseConsole):
    def __init__(self,master=None,klass=None,layout=PlotLayout,text=None,cnf={},**kw):
        Tkinter.LabelFrame.__init__(self,master,cnf,**kw)
        self.sync = False
        if text == None: text = self._name
        self.configure(text=text)
        self.klass = klass
        #self.layout = weakref.ref(layout(self,klass=klass))
        layout(self,klass=klass,name='layout')
        self.layout().pack(expand=True,fill=Tkinter.BOTH)
        self.make_menu(klass)
    def make_menu(self,cls):
        try: cls.Menu
        except AttributeError:
            klass = DefaultMenu
        else:
            klass = cls.Menu
        menu = klass(master=self,name='menu')
        # PyLAFメニューの削除
        index = menu.labeltoindex('PyLAF')
        if not index == None:
            menu.delete(index)
        # ポップアップイベントのバインド
        self.bind('<Control-Button-1>', self._rclicked) # Ctrl+右クリック
        if platform.system() == 'Darwin': # Darwinでは右クリックが異なる
            self.bind('<Button-2>', self._rclicked) # DarwinだったらB2
        else:
            self.bind('<Button-3>', self._rclicked) # Windows,LinuxだったらB3
    def _rclicked(self,e):
        self.menu().tk_popup(e.x_root,e.y_root)
    def destroy(self):
        if self.sync and (not self.component() == None):
            self.component().destroy()
        Tkinter.LabelFrame.destroy(self)
    def sync_on(self):
        self.sync = True
        
if __name__ == '__main__':
    import core
    from kitchen import Kitchen # gui内部からkitchenは機能しない
    '''
    テストコード
    '''
    tk = Tkinter.Tk()
    # テストコンポーネント
    class Test(core.Component):
        def __init__(self,master=None,name=None):
            core.Component.__init__(self,master,name)
            self.parametera = core.Port(0.)
            self.parameterb = core.Port(10.)
            self.parameterc = core.Port(100.)
            embed = EmbedTest(self,name='sigout')
            embed.sigin.set('updated')
        class Plot(Panel):
            def __init__(self,master=None,cnf={},**kw):
                Panel.__init__(self,master,cnf,**kw)
                Embed(self,klass=EmbedTest,layout=PlotLayout,name='sigout').grid(row=0,column=0,sticky=Tkinter.E+Tkinter.W+Tkinter.N+Tkinter.S)
                self.grid_rowconfigure(0,weight=1)
                self.grid_columnconfigure(0,weight=1)
        class Control(Panel):
            def __init__(self,master=None,cnf={},**kw):
                Panel.__init__(self,master,cnf,**kw)
                b, l, e = TableBuilder(self), Tkinter.Label, widgets.Entry
                b.add(l,text='Parameter A').grid().add(e,name='parametera').grid()
                b.add(l,text='Parameter B').grid().add(e,name='parameterb').grid()
                b.add(e,name='parameterc').grid(columnspan=2)
                b.add(l,text='Parameter D').grid().add(e,name='parameterd').grid()
                b.add(e,name='parametere').grid(columnspan=2)
                #
                self.linkmap = {
                #                'parametera'       : None,
                #                'parameterb'       : None,
                #                'parameterb.value' : 'parametera',
                                'parameterb'       : 'parametera',
                                }
                #
                e(self.widget('parametere'),name='parameterea')
                e(self.widget('parametere.parameterea'),name='parametereaa')
                #
                b = Panel(self,name='panelb')
                c = Panel(b,name='panelc')
                d = Tkinter.Frame(self)
                e = Panel(d,name='panele')
                f = Panel(d,name='panelf')
                g = Tkinter.Frame(f)
                h = Panel(g,name='panelh')
        class Config:
            class Decolation(Panel):
                def __init__(self,master=None,cnf={},**kw):
                    Panel.__init__(self,master,cnf,**kw)
                    builder = TableBuilder(self)
                    builder.add(Tkinter.Label,text='Parameter A').grid()
                    builder.add(widgets.Entry,name='parametera').grid()
                    builder.add(Tkinter.Label,text='Parameter B').grid()
                    builder.add(widgets.Entry,name='parameterb').grid()
                    builder.add(widgets.Entry,name='parameterc').grid(columnspan=2)
                    builder.add(Tkinter.Label,text='Parameter D').grid()
                    builder.add(widgets.Entry,name='parameterd').grid()
                    self.port('parameterd').bind(self._parameterd)
                    self.linkmap = {}
                def _parameterd(self):
                    value = self.port('parameterd').get()
                    self.sibling('control').port('parameterd').set(value)
    class EmbedTest(core.Component):
        def __init__(self,master=None,name=None):
            core.Component.__init__(self,master,name)
            self.a     = core.Port('a')
            self.b     = core.Port('b')
            self.c     = core.Port('c')
            self.sigin = core.Port('sigin')
        class Plot(Panel):
            def __init__(self,master=None,cnf={},**kw):
                Panel.__init__(self,master,cnf,**kw)
                builder = TableBuilder(self)
                builder.add(Tkinter.Label,text='Parameter A').grid()
                builder.add(widgets.Entry,name='a').grid()
                builder.add(Tkinter.Label,text='Parameter B').grid()
                builder.add(widgets.Entry,name='b').grid()
                builder.add(widgets.Entry,name='c').grid(columnspan=2)
                builder.add(widgets.Entry,name='sigin').grid(columnspan=2)
        class Control(Panel):
            def __init__(self,master=None,cnf={},**kw):
                Panel.__init__(self,master,cnf,**kw)
                builder = TableBuilder(self)
                builder.add(Tkinter.Label,text='Parameter A').grid()
                builder.add(widgets.Entry,name='a').grid()
                builder.add(Tkinter.Label,text='Parameter B').grid()
                builder.add(widgets.Entry,name='b').grid()
    #
    console = Console(tk,Test())
    console.master.title('console')
    #
    Tkinter.mainloop()
    # TODO: setattr(component,uuidname,port)で新しいポートを作って、リンクした後にdel uuidnameすればいい


# coding: utf-8
'''
PyLAF.coreを動作させるのに最低限必要なモジュール
'''

def itemcopy(item):
    try:
        item.copy
    except AttributeError:
        return item
    else:
        return item.copy()

from singleton import *
from observer import *



class Singleton(type):
    # http://code.activestate.com/recipes/412551/
    def __init__(self,*args):
        type.__init__(self,*args)
        self._instances = {}
    def __call__(self,*args):
        if not args in self._instances:
            self._instances[args] = type.__call__(self,*args)
        return self._instances[args]



# coding: utf-8

#import weakref, inspect, platform, types, time
import re, functools, threading, Queue, weakref
import utils.gui, core#, utils.functions
#from functools import partial
from utils.gui.tkwidgets import *
from scipy import log10

def classname(obj): return repr(obj.__class__).split('.')[-1].split(' ')[0]

class Projection:
    class Through:
        @classmethod
        def forward(cls,x):  return x
        @classmethod
        def backward(cls,x): return x
    class INV:
        @classmethod
        def forward(cls,x):  return 1./x
        @classmethod
        def backward(cls,x): return 1./x
    class DB:
        @classmethod
        def forward(cls,x):  return 10.*log10(x)
        @classmethod
        def backward(cls,x): return 10.**(x/10.)

class Bind:
    class Callback:
        def __init__(self):
            self.callbacks = []
        def __call__(self,*args,**kwargs):
            for callback in self.callbacks:
                callback(*args,**kwargs)
        def bind(self,callback):
            self.callbacks.append(callback)
    def bind(self,event,callback):
        try: self.bind_callbacks
        except AttributeError:
            self.bind_callbacks = {}
        if not self.bind_callbacks.has_key(event):
            cb = Bind.Callback()
            self.bind_callbacks[event] = cb
            Tkinter.Widget.bind(self,event,cb)
        self.bind_callbacks[event].bind(callback)
        
class Entry(Bind,Tkinter.Entry):
    def __init__(self,master=None,value='',format='%s',range=None,projection=Projection.Through,cnf={},**kw):
        Tkinter.Entry.__init__(self,master,cnf,**kw)
        self.projection = projection
        self.format = format
        self.old = None
        self.range = range
        self.value = core.Port(None).bind(self._forward)
        self.value.set(value)
        self.bind('<Escape>',self._forward) # ESCキーが押されたら値を破棄する
        self.bind('<Return>',self._backward) # RETURNキーが押されたら値を確定する
        self.bind('<FocusOut>',self._forward) # FOCUSが失われたら値を確定する
    def _forward(self,*args):
        self.delete(0,len(self.get()))
        self.insert(0,self.format % self.projection.forward(self.value.get()))
    def _backward(self,*args):
        old = self.projection.forward(self.value.get())
        try: # 文字列を変換できればその値を
            new = type(self.value.get())(self.get())
        except ValueError: # できなければ零を代入する
            new = type(self.value.get())('0')
        if self.range:
            if not self.range(new): new = old # もし範囲に入っていなければ以前の値に戻す
        self.value.set(self.projection.backward(new)) # Portで保持しているオブジェクト型に変換してセット
            
class EntryTable(Tkinter.Frame):
    def __init__(self,master=None,row=1,column=1,format='%s',rowlabel=None,columnlabel=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.value = core.Port([['' for c in range(column)] for r in range(row)]).bind(self._forward)
        self.joint = []
        self.row   = row
        self.column = column
        # カラムラベルの生成
        if columnlabel:
            if type(columnlabel) == str:
                for j in range(column): Label(self,text=re.sub('%d','%d' % (j+1),columnlabel)).grid(row=0,column=j+1)
            else:
                for j,label in enumerate(columnlabel): Label(self,text=label).grid(row=0,column=j+1)
        # ローラベルの生成
        if rowlabel:
            if type(rowlabel) == str:
                for i in range(row): Label(self,text=re.sub('%d','%d' % (i+1),rowlabel)).grid(row=i+1,column=0)
            else:
                for i,label in enumerate(rowlabel): Label(self,text=label).grid(row=i+1,column=0)
        # エントリーウィジェットの生成
        for i in range(row):
            joint = []
            for j in range(column):
                #print i,j
                joint.append(core.Port('').bind(functools.partial(self._backward,i=i,j=j)))
                #w = Entry(self,name='item%d%d' % (i,j),format=format)
                w = Entry(self,format=format)
                w.grid(row=i+1,column=j+1,sticky=Tkinter.W+Tkinter.E)
                joint[-1].link(w.value)
            self.joint.append(joint)
    def _forward(self):
        self._forward_called = True
        value = self.value.get()
        for i in range(self.row):
            for j in range(self.column):
                if i >= len(value) or j >= len(value[0]):
                    self.joint[i][j].set(0)
                else:
                    self.joint[i][j].set(value[i][j])
        del self._forward_called
    def _backward(self,i,j):
        try: self._forward_called
        except AttributeError:
            value = self.value.get()
            if i < len(value) and j < len(value[0]):
                value[i][j] = self.joint[i][j].get()
                self.value.set(value)
    def entryconfig(self,**kw):
        for key in self.children:
            entry = self.children[key]
            if not entry.__class__ == Entry: continue
            entry.config(**kw)
            
class MultipleEntries(Tkinter.Frame):
    def __init__(self,master=None,name=None,column=2,projection=None,frameoption={},cnf={},**kw):
        Tkinter.Frame.__init__(self,master,name=name,cnf=frameoption)
        self.value      = core.Port(['' for i in range(column)]).bind(self._forward)
        self.mode       = core.Port(None).bind(self._forward)
        self.joint      = []
        self.column     = column
        self.projection = projection
        for i in range(column):
            self.joint.append(core.Port('').bind(self._backward))
            w = Entry(self,name='item%d' % i,cnf=cnf,**kw)
            w.grid(row=0,column=i)
            self.joint[-1].link(w.value)
    def _forward(self):
        value = self.value.get()[:self.column]
        if self.projection:
            forward = self.projection[self.mode.get()][0]
            value = forward(value)
        for o,v in zip(self.joint,value): o.set(v)
    def _backward(self):
        value    = [o.get() for o in self.joint]
        if self.projection:
            backward = self.projection[self.mode.get()][1]
            value    = backward(value)
        self.value.set(value)
        
class Label(Tkinter.Label):
    def __init__(self,master=None,format='%s',cnf={},**kw):
        Tkinter.Label.__init__(self,master,cnf,**kw)
        self.format = format
        self.value  = core.Port('').bind(self.updated)
    def updated(self):
        self.configure(text=self.format % self.value.get())

class LabelTable(Tkinter.Frame):
    def __init__(self,master=None,name=None,row=1,column=1,rowlabel=None,columnlabel=None,width=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,name=name)
        self.value = core.Port([['' for c in range(column)] for r in range(row)]).bind(self._forward)
        self.joint = []
        self.row   = row
        self.column = column
        # カラムラベルの生成
        if columnlabel:
            if type(columnlabel) == str:
                for j in range(column): Label(self,text=re.sub('%d','%d' % (j+1),columnlabel)).grid(row=0,column=j+1)
            else:
                for j,label in enumerate(columnlabel): Label(self,text=label).grid(row=0,column=j+1)
        if rowlabel:
            if type(rowlabel) == str:
                rowlabel = [re.sub('%d','%d' % (i+1),rowlabel) for i in range(row)]
            for i,label in enumerate(rowlabel):
                w = Label(self,text=label)
                w.grid(row=i+1,column=0)
                w.configure(anchor=Tkinter.W)
                if width: w.configure(width=width[0])
        for i in range(row):
            joint = []
            for j in range(column):
                joint.append(core.Port(''))#.bind(self._backward))
                w = Label(self,name='item%d%d' % (i,j),cnf=cnf,**kw)
                w.grid(row=i+1,column=j+1,sticky=Tkinter.W+Tkinter.E)
                w.configure(anchor=Tkinter.W)
                if width: w.configure(width=width[j+1])
                joint[-1].link(w.value)
            self.joint.append(joint)
    def _forward(self):
        value = self.value.get()
        for i in range(self.row):
            for j in range(self.column):
                if i >= len(value) or j >= len(value[0]):
                    self.joint[i][j].set_now(0)
                else:
                    self.joint[i][j].set_now(value[i][j])
                    
class Button(Tkinter.Button):
    '''クリックすると登録したコールバックを起動する。コールバックは通常のTkinter.Buttonと同様にcommand=<method>オプションで指定できる。'''
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(None)
        self.configure(command=lambda:self.value.set(self.value.get())) # ボタンへのコールバックを設定する
        
class ToggleButton(Tkinter.Button):
    def __init__(self,master=None,text=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(0)
        self.configure(command=self._clicked)
        if not type(text) == str:
            self.configure(text=text[0])
            self.text = text
        else:
            self.configure(text=text)
    def _clicked(self):
        if (self.value.get() + 1) < len(self.text):
            value = self.value.get() + 1
        else:
            value = 0
        self.configure(text=self.text[value])
        self.value.set(value)

class TrigButton(Tkinter.Button):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(None)
        self.configure(command=self._backward)
    def _backward(self):
        self.configure(state=Tkinter.DISABLED)
        self.value.set(None)
        self.configure(state=Tkinter.NORMAL)

class LatchButton(Tkinter.Button):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Button.__init__(self,master,cnf,**kw)
        self.value = core.Port(True); self.value.bind(self._forward)
        self.configure(command=self._backward)
    def _backward(self):
        self.configure(state=Tkinter.DISABLED)
        self.value.set(False)
    def _forward(self):
        if self.value.get():
            self.configure(state=Tkinter.NORMAL)
        else:
            self.configure(state=Tkinter.DISABLED)
        
class Radiobuttons(Tkinter.Frame,utils.gui.SyncVarPort):
    def __init__(self,master=None,modes=[],cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.var = v = Tkinter.StringVar()
        for text,key in modes: Tkinter.Radiobutton(self,text=text,variable=v,value=key,command=self._backward).pack(anchor=Tkinter.W)
        self.value = core.Port(None); self.value.bind(self._forward)
        
class Checkbutton(Tkinter.Frame,utils.gui.SyncVarPort):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.var = v = Tkinter.IntVar()
        Tkinter.Checkbutton(self,variable=v,command=self._backward).pack(anchor=Tkinter.W)
        self.value = core.Port(0).bind(self._forward)

#class Menu(Tkinter.Menu):
#    TYPE_CASCADE, TYPE_ITEM = range(2)
#    ITEMS = []
#    def __init__(self,master=None,cnf={},**kw):
#        Tkinter.Menu.__init__(self,master,cnf,**kw)
#        self.callback = []
#        self.make()
#    def make(self,items=None):
#        if not items: items = self.__class__.ITEMS
#        for child in items:
#            self.add_child(self,child)
#    def remove_empty_items(self,cascade):
#        length = cascade.len()
#        for index in range(length):
#            if cascade.type(index) == 'cascade':
#                label = cascade.entrycget(index,'label')
#                menu  = cascade.nametowidget(cascade.entrycget(index,'menu'))
#                self.remove_empty_items(menu)
#                if utils.isfamily(menu,ChildrenMenu):
#                    continue
#                if menu.len() == 0:
#                    cascade.delete(index)
#    def itemtype(self,item):
#        cls = self.__class__
#        if len(item) == 1: return cls.TYPE_CASCADE
#        if type(item[1]) == list: return cls.TYPE_CASCADE
#        if repr(type(item[1])) == '<type \'classobj\'>': return cls.TYPE_CASCADE
#        if type(item[1]) == dict: return cls.TYPE_ITEM
#    def parse_cascade(self,item):
#        if len(item) == 1: return item[0], None
#        if type(item[1]) == list: return item[0], item[1:]
#        if repr(type(item[1])) == '<type \'classobj\'>': return item[0], item[1]
#    def add_child(self,parent,child):
#        def nocommand(command=None,**kw): return kw
#        cls = self.__class__
#        itemtype = self.itemtype(child) # child がどのような要素か調べる
#        if itemtype == cls.TYPE_CASCADE: # child が カスケード であるならば
#            label, item = self.parse_cascade(child) # label と item を抽出して
#            index = self.labeltoindex(label)
#            if index == None: # label が 不在 ならば
#                if not repr(type(item)) == '<type \'classobj\'>':
#                    cascade = Menu(parent,name=label.lower()) # あたらしいカスケード要素を生成して
#                    parent.add_cascade(label=label,menu=cascade)
#                    parent = cascade # 生成したカスケードを parent とする
#            else:
#                old = self.item(label)
#                if old == None: # old が カスケード でなければ
#                    self.delete(index) # 元の要素を削除して
#                    if not repr(type(item)) == '<type \'classobj\'>':
#                        cascade = Menu(parent,name=label.lower()) # 新しいカスケード要素で上書き
#                        parent.add_cascade(label=label,menu=cascade)
#                        parent = cascade
#                    else:
#                        pass
#                else:
#                    parent = old
#            if   item == None:
#                pass
#            elif repr(type(item)) == '<type \'classobj\'>':
#                parent.add_cascade(label=label,menu=item(parent,name=label.lower()))
#            elif type(item) == list:
#                for o in item:
#                    self.add_child(parent,o)
#        elif itemtype == cls.TYPE_ITEM:
#            name, kwargs = child
#            if kwargs.has_key('command'): # もしコマンドが定義されていて
#                if type(kwargs['command']) == str: # オペランドがメソッド名ならば
#                    command = getattr(self,kwargs['command']) # 自身のメソッドをコマンドに割り当てる
#                    getattr(parent,name)(command=command,**nocommand(**kwargs))
#                    return
#            getattr(parent,name)(**kwargs)
#    def assign(self,component):
#        self.comp = weakref.ref(component)
#        for key in self.children:
#            child = self.children[key]
#            if utils.functions.isfamily(child,Menu):
#                child.assign(component)
#    def component(self,path=None):
#        if not path: return self.comp()
#        ids = path.split('.')
#        child = self.comp()
#        for id in ids:
#            child = child.children[id]
#        return child
#    def labeltoindex(self,label):
#        index = 0
#        while index == self.index(index):
#            if label == self.entrycget(index,'label'):
#                return index
#            index = index + 1
#        return
#    def len(self):
#        index = 0
#        while index == self.index(index):
#            index = index + 1
#        return index
#    def item(self,label):
#        index = self.labeltoindex(label)
#        if not index: return
#        return self.nametowidget(self.entrycget(index,'menu'))

#class ChildrenMenu(Menu):
#    def assign(self,component):
#        Menu.assign(self,component)
#        self.update_children()
#    def update_children(self):
#        try: self.comp # もしコンポーネントが割り付けられていなければなにもしない
#        except AttributeError: return
#        component = self.component()
#        if component == None: # もしコンポーネントが廃棄されていれば割り付けを解除する
#            del self.comp
#            return
#        children = component.children.copy()
#        # メニューに存在する子Ｃが実在するかチェックしてメニューを削除する
#        existence = ['%s:%s' % (key,classname(children[key])) for key in children]
#        remove = []
#        for index in range(self.len()):
#            label = self.entrycget(index,'label')
#            flag = False
#            for s in existence:
#                if s == label:
#                    flag = True
#                    break
#            if not flag:
#                remove.append(index)
#        for index in remove:
#            self.delete(index)
#        # ツリーに存在する子Ｃがメニューアイテムに存在するかチェックしてメニューアイテムを追加する
#        for s in existence:
#            flag = False
#            for index in range(self.len()):
#                label = self.entrycget(index,'label')
#                if s == label:
#                    flag = True
#                    break
#            if flag: continue
#            self.add_command(label=s,command=partial(self.popup,label=s))
#        self.after(500,self.update_children)
#    def popup(self,label):
#        key = label.split(':')[0]
#        object = self.component(key)
#        #
#        tk = self
#        while tk: tk = tk.master
#        Layout.Application(Tkinter.Toplevel(tk),object,sync=False)
        
#class FileMenu(Menu):
#    ITEMS = []
#    RESTS = [
#             ['add_command', utils.kw(label='Config',command='popup_config')],
#             ['add_command', utils.kw(label='Console',command='popup_console')],
#             ]
#    def make_rest(self):
#        cls = self.__class__
#        if not self.len() == 0:
#            self.add_separator()
#        self.make(cls.RESTS)
#    def popup_config(self):
#        component = self.component()
#        if self.children.has_key('config'): return # すでにconfigを生成していなく
#        try: component.Config # componentにConfigクラスが存在していれば
#        except AttributeError: pass
#        else:
#            component.Config(master=Tkinter.Toplevel(self,name='config'),name='config').pack() # configパネルをポップアップする
#            self.children['config'].children['config'].assign(component)
#    def popup_console(self):
#        component = self.component()
#        if self.children.has_key('console'): return # すでにconsoleを生成していなく
#        try: component.Console # componentにConsoleクラスが存在していれば
#        except AttributeError: pass
#        else:
#            component.Console(master=Tkinter.Toplevel(self,name='console'),name='console').pack() # configパネルをポップアップする
#            self.children['console'].children['console'].assign(component)

#class DefaultMenu(Menu):
#    DEFAULT_ITEMS = [
#                     ['PyLAF', ],
#                     ['File', FileMenu],
#                     ['Edit'],
#                     ['Children', ChildrenMenu],
#                     ]
#    ITEMS = []
#    #RESTS = [
#    #         ['Window', ],
#    #         ['Help', ],
#    #         ]
#    # TODO:GUI毎に管理しなければならないパラメータの取り扱い。Componentからは初期値のみ管理。
#    def __init__(self,master=None,desc=[],cnf={},**kw):
#        cls = self.__class__
#        ITEMS = cls.ITEMS
#        cls.ITEMS = []
#        Menu.__init__(self,master,cnf,**kw)
#        self.make(cls.DEFAULT_ITEMS)
#        self.make(ITEMS)
#        self.make_rest(self.children)
#        self.remove_empty_items(self)
#    def make_rest(self,children):
#        #self.make(self.__class__.RESTS)
#        for key in children:
#            child = children[key]
#            if utils.isfamily(child,Menu):
#                try: make_rest = getattr(child,'make_rest')
#                except AttributeError: continue
#                make_rest()
#                self.make_rest(child.children)
                
class Thread(threading.Thread):
    EVENT_START, EVENT_CANCEL, EVENT_DONE,\
    STATE_DONE, STATE_COMPUTING, STATE_CANCELED, STATE_RUNNING = range(7)
    class StateCanceled(Exception): pass
    class SafeBuffer(Tkinter.Frame):
        def __init__(self,master=None,interval=200,cnf={},**kw):
            Tkinter.Frame.__init__(self,master,cnf,**kw)
            self.receive   = core.Port(None).bind(self._set)
            self.send      = core.Port(None)
            self.interval  = core.Port(interval)
            self._last     = None
            self._polling()
        def _set(self):
            self._last = self.receive.get()
        def _polling(self):
            if not self._last == None:
                self.send.set(self._last)
                self._last = None
            self.after(self.interval.get(),self._polling)
    class SafeQueue(Tkinter.Frame):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Frame.__init__(self,master,cnf,**kw)
            self.receive = core.Port(None).bind(self._set)
            self.send    = core.Port(None)
            self._queue  = Queue.Queue()
            self._polling()
        def _set(self):
            self._queue.put(self.receive.get())
        def _polling(self):
            try:
                while 1:
                    self.send.set(self._queue.get_nowait())
            except Queue.Empty: pass
            self.after(200,self._polling)
    class LatchedTriggerButton(Tkinter.Button):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Button.__init__(self,master,cnf,**kw)
            self.thread  = core.Port(None)
            self.remote  = core.Port(None).bind(self._forward)
            self._thread = threading.Thread(target=self._run)
            self._state  = False
            self.configure(command=self._backward)
        def _run(self):
            self.thread.set(Thread.EVENT_START)
            self.remote.set(Thread.EVENT_DONE)
        def _backward(self):
            if not self._state: self.activate()
            else              : self.deactivate()
        def _forward(self):
            remote = self.remote.get()
            if   remote == Thread.EVENT_DONE:   self.deactivate()
            elif remote == Thread.EVENT_START:  self.activate()
            elif remote == Thread.EVENT_CANCEL: self.deactivate()
        def activate(self):
            self.configure(state=Tkinter.DISABLED)
            self._state = True
            self._thread.start()
        def deactivate(self):
            self.configure(state=Tkinter.NORMAL)
            self._state = False
            self._thread = threading.Thread(target=self._run)
    class TriggerButton(Tkinter.Button):
        def __init__(self,master=None,text=['Waiting','Running'],cnf={},**kw):
            Tkinter.Button.__init__(self,master,cnf,**kw)
            self.thread  = core.Port(None)
            self.cancel  = core.Port(None)
            self.remote  = core.Port(None).bind(self._forward)
            self._thread = threading.Thread(target=self._run)
            self._state  = False
            self._text   = text
            self.configure(command=self._backward)
            self.configure(text=text[0])
        def _run(self):
            self.thread.set(Thread.EVENT_START)
            self.cancel.set(Thread.EVENT_DONE)
            self.remote.set(Thread.EVENT_DONE)
        def _backward(self):
            if not self._state: self.activate()
            else:
                self.cancel.set(Thread.EVENT_CANCEL)
                self.deactivate()
        def _forward(self,remote=None):
            if remote == None: remote = self.remote.get()
            if   remote == Thread.EVENT_DONE:  self.deactivate()
            elif remote == Thread.EVENT_START: self.activate()
            elif remote == Thread.EVENT_CANCEL:
                self.cancel.set(Thread.EVENT_CANCEL)
                self.deactivate()
        def activate(self):
            self.configure(text=self._text[1])
            self._state = True
            self._thread.setDaemon(True)
            self._thread.start()
            self.cancel.set(Thread.STATE_RUNNING)
        def deactivate(self):
            self.configure(text=self._text[0])
            self._state = False
            self._thread = threading.Thread(target=self._run)
        


# coding: utf-8

import weakvaluelist

class Observer:
    '''Subjectの監視者。単一のSubjectを監視する。メンバ変数による参照禁止'''
    def __init__(self,subject):
        self.subject = None # 監視対象を保持するための参照
        self.register(subject) # 監視対象の登録
    def register(self,subject):
        self.unregister() # 監視を解除
        subject.register(self) # 監視していることを通知
        self.subject = subject # 監視対象を保持するために参照する
    def unregister(self):
        if not self.subject == None: # すでになにかを監視しているならば
            self.subject.unregister(self) # 監視をやめることを通知
            self.subject = None # 監視対象保持のための参照を削除
    def update(self,*args,**kw): pass # 監視対象が変化したときに起動される抽象メソッド

class Subject:
    '''監視対象。いずれからも監視されなくなったら自動的に削除される。メンバ変数による参照禁止'''
    def __init__(self):
        self.observers = weakvaluelist.WeakValueList() # オブザーバとの循環参照を防ぐため弱参照のリストを格納に使う
    def register(self,observer):
        if not self.observers.count(observer): # もし、observerの弱参照を保持していなければ
            self.observers.append(observer) # オブザーバの弱参照をリストに保持
    def unregister(self,observer):
        self.observers.cleanup() # 削除したオブジェクトをコールしないよう空になった弱参照をクリア
        self.observers.remove(observer) # 与えられたオブザーバの弱参照をリストから削除
    def notify(self,*args,**kw):
        '''監視者へ変更を通知する。Observer.update()を起動する。起動の順番は登録順。'''
        self.observers.cleanup() # 削除したオブジェクトをコールしないよう空になった弱参照をクリア
        for o in self.observers.tolist(): o.update(*args,**kw) # オブザーバひとつひとつのupdate()をobserversの順番に起動

import weakref
if __name__ == '__main__':
    '''Test Code'''
    ''' いずれからも監視されなくなったらSubjectが解放されることを確認する'''
    s1 = Subject() # Subjectを生成する
    o1a = Observer(s1) # s1の監視者を生成
    o1b = Observer(s1) # s1の監視者を生成
    ws1 = weakref.ref(s1); s1  = None # Subjectへの参照s1を削除する
    print ws1() # このとき、o1a, o1bからの監視によりSubjectは保持されている
    o1a, o1b = None, None # Observerへの参照o1a, o1bを削除する
    print ws1() # いずれからも監視されなくなったSubjectは解放されている



# coding: utf-8

import utils.core, weakref, inspect, threading

def kw(**kwargs): return kwargs
def args(*args,**kwargs): return (args,kwargs)

def cb2str(callback):
    return '(%s.%s)' % (callback.weakref_obj(), callback.method_name)

def cwl(wrappers):
    string = '['
    for wrapper in wrappers:
        string = string + ' %s' % cb2str(wrapper)
    string = string + ' ]'
    return string

def isPort(obj):
    return utils.isfamily(obj,Port)

def get_child(ins,id):
    id = id.split('.')
    obj = ins
    for name in id: obj = obj.children[name]
    return obj
    
def get_port(ins,id):
    idl = id.split('.')
    id, name = idl[0], idl[-1]
    for o in idl[1:-1]: id = id + '.' + o
    return getattr(get_child(ins,id),name)
    
EVENT_SET, EVENT_GET = range(2)
HISTORY_CALLBACK = []

class CallWrapper:
    def __init__(self,method):
        self.weakref_obj = None
        self.method_name = None
        self.function    = None
        try: method.__self__ # もしも関数や部分引数オブジェクトだったら
        except AttributeError: # もし関数だったら
            self.function = method
        else: # それ以外だったら
            # obj: インスタンス, name: メソッド名
            obj, name = method.__self__, method.__name__
            self.weakref_obj = weakref.ref(obj)
            self.method_name = name
    def ismethod(self):
        return (not self.weakref_obj == None) and (self.function == None)
    def isfunction(self):
        return (self.weakref_obj == None) and (not self.function == None)
    def isalive(self):
        if   self.ismethod():
            # 参照可能か？インスタンスが生きているか？
            try: o = self.weakref_obj()
            except TypeError: return False
            # メソッドにアクセス可能か？
            try: getattr(o,self.method_name)
            except AttributeError: return False
        elif self.isfunction():
            return True
        else: return False
        return True
    def __call__(self,*args,**kwargs):
        if self.isalive():
            if   self.ismethod():
                o, name = self.weakref_obj(), self.method_name
                return getattr(o,name).__call__(*args,**kwargs)
            elif self.isfunction():
                return self.function(*args,**kwargs)
    def __eq__(self,other):
        try: o, q = self.weakref_obj(), other.weakref_obj()
        except TypeError: return False
        return (o == q) and (self.method_name == other.method_name) and (self.function == other.function)
    
class Scheduler:
    __metaclass__ = utils.core.Singleton
    def __init__(self):
        self.threads = weakref.WeakKeyDictionary()
        self.trace   = False
    def add(self,thread):
        self.threads[thread] = Schedule()
    def get(self,thread):
        return self.threads[thread]
        
class Schedule:
    def __init__(self):
        self.queue  = None
        self.ignore = []
        self.fixed  = []
    def flush(self):
        # コールバックを単一化する
        #print threading.currentThread()
        callbacks = []
        for callback in self.queue:
            if not callbacks.count(callback):
                callbacks.append(callback)
        #
        self.queue = [] # Next Queue をクリアする
        #
        for callback in callbacks:
            if not self.ignore.count(callback): # 最初の実行ならば
                if Scheduler().trace:
                    print 'Call:(%s.%s)' % (callback.weakref_obj(), callback.method_name)
                self.ignore.append(callback) # 実行フラグを立てて
                callback.__call__() # コールバックを実行し
        if not len(self.queue) == 0:
            self.flush()
            
class Root:
    __metaclass__ = utils.core.Singleton
    def __init__(self):
        self.children = {}
        self.queue  = None
        self.ignore = [] # method
        self.fixed  = [] # subject
        self.depth  = 0
        self.trace  = False
    def destroy(self):
        for c in self.children.values(): c.destroy()
    def flush(self):
        # コールバックを単一化する
        callbacks = []
        for callback in self.queue:
            if not callbacks.count(callback):
                callbacks.append(callback)
        #
        self.queue = [] # Next Queue をクリアする
        #
        # print 'Queue:', self.depth, cwl(callbacks)
        for callback in callbacks:
            if not self.ignore.count(callback): # 最初の実行ならば
                if self.trace: print 'Call:(%s.%s)' % (callback.weakref_obj(), callback.method_name)
                self.ignore.append(callback) # 実行フラグを立てて
                callback.__call__() # コールバックを実行し
                # print 'Ignored:', self.ignore.count(callback), cwl(self.ignore)
        # print 'NextQueue:', cwl(self.queue)
        if not len(self.queue) == 0:
            self.depth += 1
            self.flush()
            self.depth -= 1
        # print 'Finish:', self.depth
        
class Port(utils.core.Observer):
    def update(self,event,*args): # 監視しているストレージが更新したら直接起動されるメソッド。
        if event == EVENT_GET: return # もしGETイベントならばなにもしない
        callbacks = list(self._callbacks)
        for callback in callbacks:
            if not callback.isalive(): # コールバックがすでに廃棄済みならば、リストから削除して次のコールバックを処理する
                self._callbacks.remove(callback)
                continue
            callback.__call__() # コールバックを実行し
    def __init__(self,*args,**kw):
        if utils.isfamily(args[0],Port): # 引数がポート一族であれば
            s = Storage(args[0].storage.value) # ポートのデータを初期値として新規サブジェクトを生成し
            utils.core.Observer.__init__(self,s) # サブジェクトを監視するオブザーバを生成する
        else: # 引数がその他であれば
            s = Storage(*args,**kw) # 引数を初期値としてサブジェクトを生成し
            utils.core.Observer.__init__(self,s) # サブジェクトを監視するオブザーバを生成する
        self._callbacks = [] # コールバックを保持するタプル(obj,name)を生成する。
        self._iscaller = False
    def bind(self,method): # 監視しているストレージが更新したら起動したいメソッドを登録する。
        self._callbacks.append(CallWrapper(method))
        return self
    def unbind(self,method):
        self._callbacks.remove(CallWrapper(method))
    def unbind_all(self):
        callbacks = list(self._callbacks)
        for callback in callbacks: self._callbacks.remove(callback)
    def passive(self):
        self._deposit = self._callbacks
        self._callbacks = []
    def disable(self,target=None):
        callbacks = self._callbacks
        self._deposit = self._callbacks
        self._callbacks = []
        for callback in callbacks:
            if not callback.weakref_obj() == target and not target == None:
                self._callbacks.append(callback)
    def active(self):
        try: self._deposit
        except AttributeError: return
        for callback in self._callbacks: self._deposit.append(callback)
        self._callbacks = self._deposit
        del self._deposit
    enable = active
    def isoutput(self,klass):
        family = inspect.getmro(klass)
        for c in family:
            name = repr(c).split()[1]
            if name == 'PyLAF.core.Output' or name == 'core.Output':
                return True
        return False
    def link(self,*ports,**kw):
        # 空の弱参照をクリーンアップ
        for port in ports: # リンク対象のポートについてそれぞれ
            port.subject.observers.cleanup() # ポートが参照しているストレージの監視リストのdead参照を掃除し
        # リンクするアウトプット数を調べる
        outputs = []
        for obj in self.subject.observers.tolist():
            if self.isoutput(obj.__class__):
                outputs.append(obj)
        for port in ports: # リンク対象のポートについてそれぞれ
            for obj in port.subject.observers.tolist(): # ストレージを監視するポートすべてについて
                if self.isoutput(obj.__class__):
                    outputs.append(obj)
        if len(outputs) > 1:
            raise ValueError('too many Output was linked')
        # セットオプションの読み出し
        def _kw(set=False): return set
        set = _kw(**kw)
        for port in ports: # リンク対象のポートについてそれぞれ
            #if set: port.set(self.get())
            if set: port.set(self.subject.value)
            for obj in port.subject.observers.tolist(): # ストレージを監視するポートすべてについて
                obj.register(self.subject) # 自身の監視するポートを監視させる
        return self
    def revlink(self,port,**kw): # 逆方向リンク（オペランドのデータベースを残したリンク）
        port.link(self,**kw)
    def unlink(self):
        self.unregister()
        self.register(Storage(None))
    def set(self,*args,**kw):
        # Tk.Widgetとの同期などの際に、循環呼び出しが発生しないように
        try: self._set_called # 呼び出し元フラグが存在しなければ
        except AttributeError:
            self._set_called = None # 呼び出し元フラグを生成し
            self.subject.set(*args,**kw) # ストレージに値を設定しコールバックを呼び出し
            del self._set_called # 呼び出し元フラグを削除する
    def set_now(self,*args,**kwargs):
        self._setnow_called = None
        schedule = Scheduler().get(threading.currentThread())
        escape   = (schedule.queue, schedule.ignore, schedule.fixed)
        schedule.queue  = None
        schedule.ignore = list(schedule.ignore)
        schedule.fixed  = list(schedule.fixed)
        # set_now の最初だけ fixed を無視して書き込む
        if schedule.fixed.count(self.subject): schedule.fixed.remove(self.subject)
        #print 'Queue:', Root().queue
        #print 'Ignore:', cwl(Root().ignore)
        #print 'Fixed:', Root().fixed
        self.set(*args,**kwargs)
        #print 'Queue:', Root().queue
        #print 'Ignore:', cwl(Root().ignore)
        #print 'Fixed:', Root().fixed
        schedule.queue, schedule.ignore, schedule.fixed = escape
        del self._setnow_called
    def get(self):
        return self.subject.get()
        
class Parm(Port): # 幅優先実行のポート
    def update(self,event,*args): # 監視しているストレージが更新したら直接起動されるメソッド。
        if event == EVENT_GET: return # もしGETイベントならばなにもしない
        callbacks = list(self._callbacks)
        for callback in callbacks:
            if not callback.isalive(): # コールバックがすでに廃棄済みならば、リストから削除して次のコールバックを処理する
                self._callbacks.remove(callback)
                continue
            try: self._setnow_called
            except AttributeError: # self._setnow_calledが未定義ならば
                schedule = Scheduler().get(threading.currentThread())
                if not schedule.queue.count(callback):
                    schedule.queue.append(callback) # コールバックをキューに登録する
            else: # 定義されていれば
                callback.__call__() # コールバックを実行する
        
class Output(Port): # PULL型のポート
    def update(self,event,*args): # 監視しているストレージが更新したら直接起動されるメソッド。
        if event == EVENT_SET: return # もしSETイベントならばなにもしない
        schedule = Scheduler().get(threading.currentThread())
        if schedule.fixed.count(self.subject): return # もし確定済みならばなにもしない
        callbacks = list(self._callbacks)
        for callback in callbacks:
            if not callback.isalive():
                self._callbacks.remove(callback)
                continue
            if not schedule.ignore.count(callback): # 最初の実行ならば
                if Scheduler().trace: self.print_callback(callback)
                schedule.ignore.append(callback) # 実行フラグを立てて
                callback.__call__() # コールバックを実行し
                # print 'Ignored:', Root().ignore.count(callback), cwl(Root().ignore)
        if Scheduler().trace: self.print_queue()
    def print_callback(self,callback):
        print 'Get:(%s.%s)' % (callback.weakref_obj(), callback.method_name),
    def print_queue(self):
        schedule = Scheduler().get(threading.currentThread())
        if schedule.queue == None: return
        callbacks = []
        for callback in schedule.queue:
            if not callbacks.count(callback):
                callbacks.append(callback)
        for callback in schedule.ignore:
            if callbacks.count(callback):
                callbacks.remove(callback)
        print 'Queue:', cwl(callbacks),
        
class Storage(utils.core.Subject):
    '''
    複数のPortクラスで同期する任意のリテラルを格納する実体クラス
    メンバ変数による参照禁止
    '''
    def __init__(self,*args,**kw): # 与えられた値を保持するストレージを生成する
        utils.core.Subject.__init__(self)
        self._set(*args,**kw)
    def _set(self,*args,**kw):
        # self.value = value.copy() # もしvalueがcopyメソッドを持っていればvalue.copy()する
        # ！！！listはcopyメソッドを持たないから、Portの実装に影響するかも！！！
        if args and kw:
            args = tuple([utils.core.itemcopy(item) for item in args]) # タプルの中身をコピーする（１階層のみ）
            temp = {}
            for key in kw:
                temp[key] = utils.core.itemcopy(kw[key])
            kw = temp
            self.value = (args,kw)
        elif args:
            if len(args) == 1:
                self.value = utils.core.itemcopy(args[0])
            else:
                self.value = tuple([utils.core.itemcopy(item) for item in args]) # タプルの中身をコピーする（１階層のみ）
        elif kw:
            temp = {}
            for key in kw: temp[key] = utils.core.itemcopy(kw[key])
            self.value = temp
    def set(self,*args,**kw):
        thread = threading.currentThread()
        if not Scheduler().threads.has_key(thread):
            Scheduler().add(thread)
        schedule = Scheduler().get(thread)
        if not schedule.fixed.count(self):
            schedule.fixed.append(self)
            self._set(*args,**kw)
        # else:
        #     print 'Ignored set:', self
        if schedule.queue == None:
            schedule.queue = [] # キューを生成して
            self.notify(EVENT_SET)
            # print 'Toplevel:', cwl(Root().queue)
            schedule.flush()
            schedule.queue = None
            schedule.ignore = []
            schedule.fixed  = []
        else:
            self.notify(EVENT_SET)
    def get(self):
        '''格納しているオブジェクトまたはリテラルを返す'''
        self.notify(EVENT_GET)
        return self.value

class Component:
    '''
    PyLAF.Rootを基盤とするPyLAFコンポーネント
    メンバ変数による参照禁止
    '''
    def __init__(self,master=None,name=None):
        self.children = {}
        if master == None:
            self.master = Root()
        else:
            self.master = master
        if name == None:
            self._name = repr(id(self))
        else:
            self._name = name
        self._assign_to_master()
    def _assign_to_master(self):
        if self.master.children.has_key(self._name): # master.childrenにおけるidの唯一性を保証する
            self.master.children[self._name].destroy()
        self.master.children[self._name] = self
    def _remove_from_master(self): # master.childrenから自身へのオブジェクト参照を廃棄する
        if self.master.children.has_key(self._name):
            del self.master.children[self._name]
    def destroy(self):
        ''' Destroy this and all descendants nodes.'''
        for c in self.children.values(): c.destroy() # 子オブジェクトを巡回してdestroy()をコールする
        self._remove_from_master() # 親オブジェクトのchildren辞書から自身の参照を削除する
        # ^^^ 辞書の巡回中に辞書を変更しちゃって大丈夫？
        # self.children.clear() で一気に削除するほうが行儀いいのでは？
    def set(self,**kwargs):
        for key in kwargs:
            getattr(self,key).set(kwargs[key])
    def component(self,path=None):
        if path == None: return self
        ids = path.split('.')
        child = self
        for id in ids:
            if child.children.has_key(id):
                child = child.children[id]
            else:
                return
        return child
    def port(self,path=None):
        if path == None: return self
        name = path.split('.')[-1]
        path = path[:-(len(name)+1)]
        if len(path) == 0:
            component = self
        else:
            component = self.component(path)
        if component == None: return
        try: return getattr(component,name)
        except AttributeError: return
    child = component

# class Trig:
if __name__ == "__main__":
    class Child(Component): pass
    class MyComponent(Component):
        def __init__(self,master):
            Component.__init__(self,master)
            self.port = Port(0).bind(self.callback) # Portが自身のメソッドへアクセスできかつきちんとselfが解放される
            # 子ノードがぶら下げられても、メンバ変数として保持しなければselfが解放されるが
            c = Child(self,name='child'); print c
            # === メンバ変数として保持するとselfが解放されなくなる
            # self.child = Child(self,name='child'); print self.child
            # ===
        def callback(self):
            print self.port.get(), self.children['child']
        # destroyでPortやComponentのメンバ変数を解放するのは賢いか？シンプルではない。
        # Portがメソッドを弱参照するように、master.childrenを弱参照で保持するようにすれば自動的に解放されるようになる
    c = MyComponent(Root())
    print c.port.get()
    c.port.set('aa',xaxis='aa')
    c.port.set(xaxis='aa')
    c.port.set('%d aa' % 1)
    c.port.unbind(c.callback) # バインドの解除
    # c.port.unbind_all() # バインドの解除
    c.port.set('aa',xaxis='aa')
    
    c = weakref.ref(c)
    print c
    c().destroy()
    print c
    


# coding: utf-8

import weakref, types

def popcnf(key,cnf):
    item = None
    if cnf.has_key(key):
        item = cnf[key]
        del cnf[key]
    return item

def convweak(method): # インスタンスメソッドを弱参照フォーマットに変換する関数
    class Dummy: pass # スコープを抜けたらすぐに解放されるダミーオブジェクト用のクラス
    result = (weakref.ref(Dummy()),'') # 空の参照
    if method == None: return result # もしメソッドが未定義ならば空の参照を返す
    if type(method) == tuple: # それがすでに弱参照フォーマットならば
        result = method # そのまま返す。
    if type(method) == types.MethodType: # それがメソッドならば
        result = (weakref.ref(method.__self__),method.__name__) # 弱参照フォーマットに変換する。
    return result

def kw(**kw): return kw

class SyncVarPort:
    def _backward(self,*args):
        try: self._backward_called
        except AttributeError:
            self._backward_called = True
            self.value.set(self.var.get())
            del self._backward_called
    def _forward(self):
        try: self._forward_called
        except AttributeError:
            self._forward_called = True
            self.var.set(self.value.get())
            del self._forward_called

class WithGUI:
    '''
    GUIとの寿命同期機構など、GUIとの連携に必要な機構を格納する。
    def destroy(self):
        if self._dying: return
        self._dying = True
        WithGUI.destroy(self)
        core.Component.destroy(self)
    '''
    def __init__(self):
        self._guis = weakref.WeakValueDictionary()
        self._dying = False
    def destroy(self):
        for key in self._guis:
            gui = self._guis[key]
            if gui._sync: gui.destroy()



# coding: utf-8

import Tkinter, ttk #@UnresolvedImport
import tkFileDialog
import numpy, matplotlib, inspect
import utils.mpl, core, gui, widgets, kitchen
from numpy import recarray, zeros
from scipy import array, real, imag, fft, fftpack, log10, arange
from core import Port, Output

class ModalAxesPanel(Tkinter.Frame):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        c = self.children
        
        widgets.Radiobuttons(self,modes=[('Min/Max','minmax'),('Center/Span','centerspan')],name='mode').grid(row=0,column=1)
        c['mode'].value.set('minmax')
        projection = {'minmax'    : (lambda x: x, lambda x: x),
                      'centerspan': (lambda x: [(x[0]+x[1])/2.,x[1]-x[0]],lambda x: [x[0]-x[1]/2.,x[0]+x[1]/2])}
        
        Tkinter.Label(self,text='X Axis').grid(row=1,column=0)
        widgets.MultipleEntries(self,column=2,name='xlim',projection=projection).grid(row=1,column=1)
        widgets.Button(self,text='auto',name='autoscalex').grid(row=1,column=3)
        
        Tkinter.Label(self,text='Y Axis').grid(row=2,column=0)
        widgets.MultipleEntries(self,column=2,name='ylim').grid(row=2,column=1)
        widgets.Button(self,text='auto',name='autoscaley').grid(row=2,column=3)
        
        c['mode'].value.link(c['xlim'].mode)
        
class AxesPanel(Tkinter.Frame):
    def __init__(self,master=None,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        
        Tkinter.Label(self,text='X Axis').grid(row=1,column=0)
        widgets.MultipleEntries(self,column=2,name='xlim').grid(row=1,column=1)
        widgets.Button(self,text='auto',name='autoscalex').grid(row=1,column=3)
        
        Tkinter.Label(self,text='Y Axis').grid(row=2,column=0)
        widgets.MultipleEntries(self,column=2,name='ylim').grid(row=2,column=1)
        widgets.Button(self,text='auto',name='autoscaley').grid(row=2,column=3)
        
class BasePlot(core.Component):
    def __init__(self,master=None,name=None):
        core.Component.__init__(self,master,name)
        self.sigin  = Port(array([])).bind(self._sigin)
        self.sigout = Output(array([]))#.bind(self._sigin) # フォーマットの変化で処理を開始しないため
    def _sigin(self):
        self.sigout.set(self.sigin.get())
    class Plot(utils.mpl.BasePlot,gui.BasePanel):
        def __init__(self,*args,**kw):
            utils.mpl.BasePlot.__init__(self,*args,**kw)
            self.sigin   = Port(None).bind(self._sigin)
            self.linkmap = {'sigin' : 'sigout'}
            self.connect()
        def connect(self):
            # プロット範囲更新イベントの設定
            self._callbacks = cb = {}
            cb['cid_xlim']  = self.ax.callbacks.connect('xlim_changed',self._update_xlim)
            cb['cid_ylim']  = self.ax.callbacks.connect('ylim_changed',self._update_ylim)
        def disconnect(self):
            self.ax.callbacks.disconnect(self._callbacks['cid_xlim'])
            self.ax.callbacks.disconnect(self._callbacks['cid_ylim'])
        def destroy(self):
            self.disconnect()
            utils.mpl.BasePlot.destroy(self)
        def _sigin(self,sigin=None):
            if sigin == None: sigin = self.sigin.get()
            if sigin == None: return
            args,kw = utils.parse_port_args(sigin)
            self.plot(*args,**kw)
        def plot(self,*args,**kwargs):
            self.clear()
            self.ax.plot(*args,**kwargs)
            self.canvas.show()
            self.update_idletasks() # スレッド処理でButtonに対応するにはupdate_idletasksを使う。updateではButtonがロックすることがある。
        def _update_xlim(self,e): # ax の xlim が更新されたときに control の GUI を更新する
            control = self.sibling('control')
            if control == None: return
            self._update_xlim_called = True
            control.port('xlim').set(list(self.ax.get_xlim()))
            del self._update_xlim_called
        def _update_ylim(self,e): # ax の xlim が更新されたときに control の GUI を更新する
            control = self.sibling('control')
            if control == None: return
            self._update_ylim_called = True
            control.port('ylim').set(list(self.ax.get_ylim()))
            del self._update_ylim_called
        def resize(self,dpi=None,figsize=None):
            if (dpi == None or figsize == None): return
            subplotpars = self.ax.figure.subplotpars
            left   = subplotpars.left
            right  = subplotpars.right
            bottom = subplotpars.bottom
            top    = subplotpars.top
            title  = self.ax.get_title()
            xlabel = self.ax.get_xlabel()
            ylabel = self.ax.get_ylabel()
            self.disconnect()
            utils.mpl.BasePlot.resize(self,dpi=dpi,figsize=figsize)
            self.connect()
            self.ax.set_ylabel(ylabel)
            self.ax.set_xlabel(xlabel)
            self.ax.set_title(title)
            self.ax.figure.subplots_adjust(left = left, right = right, bottom = bottom, top = top)
            if not self.sigin.subject.value == None:
                self._sigin(self.sigin.subject.value) # Portの実装に依存してしまっている！ので後で考えよう。
    class Control(gui.Panel):
        def __init__(self,master=None,cnf={},**kw):
            gui.Panel.__init__(self,master,cnf,**kw)
            builder = gui.TableBuilder(self)
            builder.add(AxesPanel,name='axes').grid(columnspan=2)
            #
            plot = self.sibling('plot')
            if plot.ax.get_autoscalex_on(): # autoscalex ならば
                self.widget('autoscalex').config(state=Tkinter.DISABLED) # ボタンを不活性
            else: # not autoscalex ならば
                self.widget('autoscalex').config(state=Tkinter.NORMAL) # ボタンを活性
            if plot.ax.get_autoscaley_on():
                self.widget('autoscaley').config(state=Tkinter.DISABLED)
            else:
                self.widget('autoscaley').config(state=Tkinter.NORMAL)
            self.port('xlim').set(list(plot.ax.get_xlim()))
            self.port('ylim').set(list(plot.ax.get_ylim()))
            #
            self.port('xlim').bind(self._changed_xlim)
            self.port('ylim').bind(self._changed_ylim)
            self.port('autoscalex').bind(self._changed_autoscalex)
            self.port('autoscaley').bind(self._changed_autoscaley)
        def _changed_xlim(self):
            '''
            xlim エントリーが変更されたらば、
            autoscalex を disable し、
            autoscalex ボタンを enable し、
            xlim を更新し、
            プロットを再描画する
            '''
            plot = self.sibling('plot')
            try: plot._update_xlim_called
            except AttributeError:
                plot.ax.set_autoscalex_on(False)
                self.widget('autoscalex').config(state=Tkinter.NORMAL)
                #plot.autoscalex_config.set(state=Tkinter.NORMAL)
                plot.ax.set_xlim(*self.port('xlim').get())
                plot.canvas.show()
        def _changed_ylim(self):
            plot = self.sibling('plot')
            try: plot._update_ylim_called
            except AttributeError:
                plot.ax.set_autoscaley_on(False)
                self.widget('autoscaley').config(state=Tkinter.NORMAL)
                plot.ax.set_ylim(*self.port('ylim').get())
                plot.canvas.show()
        def _changed_autoscalex(self):
            '''
            autoscalexボタンを押したらば、
            autoscalexをenableし、
            autoscalexボタンをdisableし、
            再プロットする
            '''
            plot = self.sibling('plot')
            plot.ax.set_autoscalex_on(True)
            self.widget('autoscalex').config(state=Tkinter.DISABLED)
            plot._sigin()
        def _changed_autoscaley(self):
            plot = self.sibling('plot')
            plot.ax.set_autoscaley_on(True)
            self.widget('autoscaley').config(state=Tkinter.DISABLED)
            plot._sigin()
    class Config:
        class Decolation(gui.Panel):
            def __init__(self,master=None,cnf={},**kw):
                gui.Panel.__init__(self,master,cnf,**kw)
                s = {'sticky' : Tkinter.W+Tkinter.E}
                b, l, e = gui.TableBuilder(self), Tkinter.Label, widgets.Entry
                b.add(l,text='PADL').grid(**s).add(e,name='padl').grid(**s)
                b.add(l,text='PADR').grid(**s).add(e,name='padr').grid(**s)
                b.add(l,text='PADB').grid(**s).add(e,name='padb').grid(**s)
                b.add(l,text='PADT').grid(**s).add(e,name='padt').grid(**s)
                b.add(widgets.Button,name='reset_pad',text='reset').grid(columnspan=2,**s)
                b.add(l,text='TITLE').grid(**s).add(e,name='title').grid(**s)
                b.add(l,text='XLABEL').grid(**s).add(e,name='xlabel').grid(**s)
                b.add(l,text='YLABEL').grid(**s).add(e,name='ylabel').grid(**s)
                b.add(widgets.Button,name='reset_label',text='reset').grid(columnspan=2,**s)
                b.add(l,text='DPI').grid(**s).add(e,name='dpi').grid(**s)
                b.add(l,text='FIGSIZE').grid(**s).add(widgets.MultipleEntries,name='figsize').grid(**s)
                #
                plot        = self.sibling('plot')
                subplotpars = plot.ax.figure.subplotpars
                self._config = {
                                'left'   : subplotpars.left,
                                'right'  : subplotpars.right,
                                'bottom' : subplotpars.bottom,
                                'top'    : subplotpars.top,
                                'title'  : plot.ax.get_title(),
                                'xlabel' : plot.ax.get_xlabel(),
                                'ylabel' : plot.ax.get_ylabel(),
                                'figsize' : plot.figure.get_size_inches(),
                                'dpi'     : plot.figure.get_dpi(),
                                }
                self.port('padl').set(subplotpars.left)
                self.port('padr').set(subplotpars.right)
                self.port('padb').set(subplotpars.bottom)
                self.port('padt').set(subplotpars.top)
                self.port('title').set(plot.ax.get_title())
                self.port('xlabel').set(plot.ax.get_xlabel())
                self.port('ylabel').set(plot.ax.get_ylabel())
                self.port('figsize').set(self._config['figsize'])
                self.port('dpi').set(self._config['dpi'])
                #
                self.port('reset_pad').bind(self._reset_pad)
                self.port('reset_label').bind(self._reset_label)
                self.port('figsize').bind(self.resize)
                self.port('dpi').bind(self.resize)
                self.port('padl').bind(self.update_adjust)
                self.port('padr').bind(self.update_adjust)
                self.port('padb').bind(self.update_adjust)
                self.port('padt').bind(self.update_adjust)
                self.port('title').bind(self.set_title)
                self.port('xlabel').bind(self.set_xlabel)
                self.port('ylabel').bind(self.set_ylabel)
            def resize(self):
                figsize = self.port('figsize').get()
                dpi     = self.port('dpi').get()
                self.sibling('plot').resize(dpi,figsize)
            def set_title(self):
                plot = self.sibling('plot')
                plot.ax.set_title(self.port('title').get())
                plot.canvas.show()
            def set_xlabel(self):
                plot = self.sibling('plot')
                plot.ax.set_xlabel(self.port('xlabel').get())
                plot.canvas.show()
            def set_ylabel(self):
                plot = self.sibling('plot')
                plot.ax.set_ylabel(self.port('ylabel').get())
                plot.canvas.show()
            def update_adjust(self):
                plot = self.sibling('plot')
                plot.ax.figure.subplots_adjust(left   = self.port('padl').get(),
                                               right  = self.port('padr').get(),
                                               bottom = self.port('padb').get(),
                                               top    = self.port('padt').get())
                plot.canvas.show()
            def _reset_pad(self):
                self.port('padl').set(self._config['left'])
                self.port('padr').set(self._config['right'])
                self.port('padb').set(self._config['bottom'])
                self.port('padt').set(self._config['top'])
            def _reset_label(self):
                self.port('title').set(self._config['title'])
                self.port('xlabel').set(self._config['xlabel'])
                self.port('ylabel').set(self._config['ylabel'])
    class Menu(gui.DefaultMenu):
        ITEMS = [
                 ['File',
                  ['add_command', utils.functions.kw(label='Import',command='_import')],
                  ['add_command', {'label' : 'Export', 'command' : '_export'}],
                  ['add_separator', {}],
                  ['add_command', {'label' : 'Clear', 'command' : '_clear'}],
                  # Export Original Data
                  # Export Processed Data
                  ],
                 ]
        def _clear(self):
            plot = self.master.panel('plot')
            plot.clear()
            plot.canvas.show()
            plot.update_idletasks() # スレッド処理でButtonに対応するにはupdate_idletasksを使う。updateではButtonがロックすることがある。
            self.master.component().sigout.set(None)
        def _import(self):
            fname = tkFileDialog.askopenfilename()
            a    = numpy.loadtxt(fname).transpose()
            #component = self.master.component()
            #component.sigout.set(self.import_format(a))
            self.master.panel('plot').plot(*self.import_format(a))
        def _export(self):
            ax    = self.master.panel('plot').ax
            lines = ax.get_lines()
            if len(lines) == 0: return
            buf   = []
            buf.append(lines[0].get_xdata())
            for line in lines:
                buf.append(line.get_ydata())
            fname = tkFileDialog.asksaveasfilename()
            if not fname == None:
                numpy.savetxt(fname,self.export_format(array(buf).transpose()))
        def import_format(self,a):
            return a[0,:], a[1:,:].transpose()
        def export_format(self,buf):
            return buf
            
class BasePlotPanel(gui.Panel):
    def __init__(self,master=None,cnf={},**kw):
        gui.Panel.__init__(self,master,cnf,**kw)
        gui.Embed(self,BasePlot,text='BasePlot').pack(side=Tkinter.LEFT)

class Oscilloscope(BasePlot):
    def __init__(self,master=None,name=None):
        BasePlot.__init__(self,master,name)
        self.xunit      = Port(1e-3)
        self.yunit      = Port(1e-3)
        self.samplerate = Port(1.)
        members = [key for key,o in inspect.getmembers(master,core.isPort)]
        if members.count('samplerate'): master.samplerate.link(self.samplerate,set=True)
    class Plot(BasePlot.Plot):
        def __init__(self,*args,**kw):
            BasePlot.Plot.__init__(self,*args,**kw)
            self.ax.set_xlabel('Time [us]')
            self.ax.set_ylabel('Signal [mV]')
            self.samplerate = Port(1.)
            self.decimation = Port(1)
        def _sigin(self,sigin=None):
            if sigin == None: sigin = self.sigin.get()
            if sigin == None: return
            sig, kw = utils.parse_port_args(sigin)
            samplerate = self.samplerate.get()
            decimation = self.decimation.get()
            sig = sig[0]
            sig = sig[...,::decimation]
            time  = arange(sig.shape[-1]) / samplerate * decimation
            BasePlot.Plot.plot(self,time,sig.transpose(),**kw)
    class Control(BasePlot.Control):
        def __init__(self,master=None,cnf={},**kw):
            BasePlot.Control.__init__(self,master,cnf,**kw)
            b, tkl, e, l = gui.TableBuilder, Tkinter.Label, widgets.Entry, widgets.Label
            b = gui.TableBuilder(self)
            b.add(tkl,text='Sample Rate [MS/sec]').grid().add(l,name='samplerate').grid()
            b.add(tkl,text='Decimation Index').grid().add(e,name='decimation').grid()
            #
            plot = self.sibling('plot')
            plot.decimation.link(self.port('decimation'),set=True)
            self.port('decimation').bind(self._trig)
        def _trig(self): self.component().sigout.set(self.component().sigout.get())
    class Config(BasePlot.Config):
        class Decolation(BasePlot.Config.Decolation):
            def __init__(self,master=None,cnf={},**kw):
                BasePlot.Config.Decolation.__init__(self,master,cnf,**kw)
                b, l, e = gui.TableBuilder(self), Tkinter.Label, widgets.Entry
                b.add(l,text='XUNIT').grid().add(e,name='xunit').grid()
                b.add(l,text='YUNIT').grid().add(e,name='yunit').grid()
#    class Menu(BasePlot.Menu):
#        def import_format(self,a):
#            dt = a[0,1] - a[0,0]
#            self.master.component().samplerate.set(dt)
#            return a[1:,:]
            
class OscilloscopePanel(gui.Panel):
    def __init__(self,master=None,cnf={},**kw):
        gui.Panel.__init__(self,master,cnf,**kw)
        gui.Embed(self,Oscilloscope,text='Oscilloscope').pack(side=Tkinter.LEFT)
        
class Gauss(BasePlot):
    #def _sigin(self):
    #    sigin = self.sigin.get()
    #    c, kw = utils.parse_port_args(sigin)
    #    c = c[0]
    #    self.sigout.set(real(c),imag(c),**kw)
    class Plot(BasePlot.Plot):
        def __init__(self,*args,**kw):
            BasePlot.Plot.__init__(self,*args,**kw)
            self.ax.set_title('Gauss Plane')
            self.ax.set_xlabel('Real(x)')
            self.ax.set_ylabel('Imag(x)')
        def _sigin(self,sigin=None):
            if sigin == None: sigin = self.sigin.get()
            if sigin == None: return
            c, kw = utils.parse_port_args(sigin)
            c = c[0]
            BasePlot.Plot.plot(self,real(c),imag(c),**kw)

class GaussPanel(gui.Panel):
    def __init__(self,master=None,cnf={},**kw):
        gui.Panel.__init__(self,master,cnf,**kw)
        gui.Embed(self,Gauss,text='Gauss').pack(side=Tkinter.LEFT)

class Spectrum(BasePlot):
    def __init__(self,master=None,name=None):
        BasePlot.__init__(self,master,name)
        self.samplerate = Port(1.)
        members = [key for key,o in inspect.getmembers(master,core.isPort)]
        if members.count('samplerate'): master.samplerate.link(self.samplerate,set=True)
    class Plot(BasePlot.Plot):
        def __init__(self,*args,**kw):
            BasePlot.Plot.__init__(self,*args,**kw)
            self.ax.set_title('Spectrum')
            self.ax.set_xlabel('Frequency [MHz]')
            self.ax.set_ylabel('Power [dBm]')
            self.samplerate = Port(1.)
            self.decimation = Port(1)
        def _sigin(self,sigin=None):
            if sigin == None: sigin = self.sigin.get()
            if sigin == None: return
            sig, kw = utils.parse_port_args(sigin)
            #if len(sig.shape) == 1: sig = sig.reshape(1,len(sig))
            samplerate = self.samplerate.get()
            decimation = self.decimation.get()
            sig = sig[0]
            sig = sig[...,::decimation]
            # sig  = sig[:2**int(log2(sig.size))]
            try:
                f   = fft(sig)
            except ValueError:
                from warnings import warn
                warn('ValueError has occur.')
                return
            # f[0] = f[0] * .5 # 片側スペクトルを見たい場合にはこの行をアクティベート
            v   = fftpack.fftshift(f * 2.0 / sig.shape[-1])
            freq = fftpack.fftshift(fftpack.fftfreq(sig.shape[-1],1./samplerate*decimation))
            value = 10 * log10(real(v*v.conj())/2/50) - 30
            BasePlot.Plot.plot(self,freq,value.transpose(),**kw)
            #self.sigout.set(freq, value.transpose())
    class Control(BasePlot.Control):
        def __init__(self,master=None,cnf={},**kw):
            BasePlot.Control.__init__(self,master,cnf,**kw)
            b, tkl, e, l = gui.TableBuilder, Tkinter.Label, widgets.Entry, widgets.Label
            b = gui.TableBuilder(self)
            b.add(tkl,text='Sample Rate [MS/sec]').grid().add(l,name='samplerate').grid()
            b.add(tkl,text='Decimation Index').grid().add(e,name='decimation').grid()
            #
            plot = self.sibling('plot')
            plot.decimation.link(self.port('decimation'),set=True)
            self.port('decimation').bind(self._trig)
        def _trig(self):
            self.component().sigout.set(self.component().sigout.get())
            
class SpectrumPanel(gui.Panel):
    def __init__(self,master=None,cnf={},**kw):
        gui.Panel.__init__(self,master,cnf,**kw)
        gui.Embed(self,Spectrum,text='Spectrum').pack(side=Tkinter.LEFT)

import weakref, scipy
if __name__ == '__main__':
    class MyComponent(core.Component):
        def __init__(self,master=None,name=None):
            core.Component.__init__(self,master,name)
            scope = Oscilloscope(self,name='scope') # メモリリークを起こしますのでメンバ変数として保持しないでください
            self.sigout = Port(array([])); self.sigout.link(scope.sigin)
            self.freq = Port(1.)
        def _trig(self):
            tim = scipy.arange(1000) / 100.
            omg = 2 * scipy.pi * self.freq.get() * tim
            self.sigout.set(scipy.sin(omg))
            self.freq.set(self.freq.get() + .1)
        class Plot(gui.Panel):
            def __init__(self,master=None,cnf={},**kw):
                gui.Panel.__init__(self,master,cnf,**kw)
                gui.Embed(self,Oscilloscope,name='scope',text='Scope1').pack(side=Tkinter.LEFT)
                gui.Embed(self,Oscilloscope,text='Scope2').pack(side=Tkinter.LEFT)
        class Control(gui.Panel):
            def __init__(self,master=None,cnf={},**kw):
                gui.Panel.__init__(self,master,cnf,**kw)
                builder = gui.TableBuilder(self)
                builder.add(Tkinter.Label,text='Frequency').grid()
                builder.add(widgets.Entry,name='freq').grid().bind('<Return>',self._trig)
                builder.add(Tkinter.Button,name='trig',text='trig',command=self._trig).grid(columnspan=2,sticky=Tkinter.W+Tkinter.E)
                #
            def _trig(self,*args): self.component()._trig()
    tk = Tkinter.Tk()
    a = MyComponent()
    a1 = gui.Console(tk,a)#; a1.assign(a); a1.pack(); 
    a1.master.title('master')
    a2 = gui.Console(Tkinter.Toplevel(tk),Oscilloscope())
    a2.master.title('osc')
    a1.component().sigout.link(a2.component().sigin)
    Tkinter.mainloop()
    import gc
    a1 = weakref.ref(a1); gc.collect(); print 'a1:', a1
    a  = weakref.ref(a);  gc.collect(); print 'a :', a
    a2 = weakref.ref(a2); gc.collect(); print 'a2:', a2
    


# coding: utf-8

import weakref

# listと完全互換で内部処理だけweakrefってのが理想なんだけど．．．結構大変そう

class WeakValueList(list):
    '''
    オブジェクトの弱参照を保持するリスト
    '''
    def count(self,x):
        return list.count(self, weakref.ref(x))
    def append(self,obj):
        '''objへの弱参照をリストへ追加する'''
        list.append(self, weakref.ref(obj))
    def insert(self,index,obj):
        '''objへの弱参照をindex番目に挿入する'''
        list.insert(self, index, weakref.ref(obj))
    def remove(self,obj):
        '''objへの弱参照のうち最初のひとつをリストから削除する'''
        '''もしobjが見つからなければエラーを返す'''
        for ref in self: # 弱参照をひとつづつ取り出し
            if obj is ref(): # もし、弱参照がobjへのものならば
                list.remove(self, ref) # リストからobjへの弱参照を削除し
                return # ループから抜け出す
        raise ValueError
    def tolist(self):
        '''弱参照のリストを参照のリストへ変換する'''
        return [ref() for ref in self]
    def cleanup(self):
        '''
        dead参照のアイテムを削除する
        '''
        items = [] # リストを正しく巡回するために良いしたdead参照を保持するためのバッファ
        for ref in self: # リストを巡回し
            if ref() == None: # dead参照を見つけたら
                items.append(ref) # バッファにためる
        for item in items: # バッファを巡回し
            list.remove(self, item) # dead参照を削除する
    
if __name__ == '__main__':
    '''Test Code'''
    '''含まれていない要素の場合、raiseされるかremoveをテストする'''
    class Dummy: pass
    abc, defg, hijkl = Dummy(), Dummy(), Dummy()
    print abc, defg, hijkl
    l = WeakValueList()
    l.append(abc); l.append(defg); l.append(hijkl)
    print l.count(defg)
    l[0] = abc
    print l, l[0]
    for c in l:
        print c



# coding: utf-8

import weakref, platform, random, uuid, inspect, imp, os
import Tkinter, tkFileDialog
import ttk #@UnresolvedImport
import core, gui, widgets, mpl

if platform.system() == 'Darwin':
    BUTTON_R = '<Button-2>'
else:
    BUTTON_R = '<Button-3>'

def classname(obj): return repr(obj.__class__).split('.')[-1].split(' ')[0]

def invoke_kitchen(master=None):
    if master.__class__ == Tkinter.Tk:
        Kitchen(master).pack()
    else:
        Kitchen(Tkinter.Toplevel(master)).pack()
def prepare():
    invoke_kitchen(Tkinter.Tk())
    
#class KitchenMenu(gui.Menu):
#    def __init__(self,master=None,desc=[],cnf={},**kw):
#        gui.Menu.__init__(self,master,cnf,**kw)
#        self.add_command(label='Kitchen',command=invoke_kitchen)
#    def assign(self,component): pass

def kw(**kw): return kw
#gui.DefaultMenu.DEFAULT_ITEMS[0].insert(1, ['Kitchen', KitchenMenu])
gui.DefaultMenu.DEFAULT_ITEMS[0].insert(1, ['add_command', kw(label='Kitchen',command=invoke_kitchen)])

def iscontain(obj,cls):
    try: bases = inspect.getmro(obj.__class__) # オブジェクトのクラスツリーをタプルで返す
    except AttributeError:
        return False
    if bases.count(cls):
        return True
    else:
        return False

class SimpleDialog(Tkinter.Toplevel): pass

class ComponentChooser(ttk.Frame):
    def __init__(self,master=None,**kw):
        ttk.Frame.__init__(self,master,**kw)
        self.klass = {}
        self.selection = None
        ttk.Button(self,text='Load Module',command=self.select_module).grid(row=0,column=0)
        ttk.Treeview(self,name='klass').grid(row=1,column=0)
        ttk.Button(self,text='Select',command=self.made_decision).grid(row=2,column=0)
        #
        self.update()
        self.select_package()
        #
        self.pack()
        self.grab_set()
        self.focus_set()
        self.wait_window(self)
    def select_package(self,dir=os.getcwd()):
        selection = []
        for name in os.listdir(dir):
            if name.endswith('.py'):
                selection.append(name)
        if len(selection) == 0: return
        self.parse_selection(selection)
    def select_module(self):
        # askopenfilename, askopenfilenames, askdirectory
        selection = tkFileDialog.askopenfilenames(initialdir=os.getcwd(),filetypes=[('python modules','*.py')])
        selection = list(selection)
        self.parse_selection(selection)
    def parse_selection(self,selection):
        item   = selection[0]
        path   = item.split('/')
        pyname = path[-1]
        cwd    = item.replace(pyname,'')
        names = []
        for item in selection:
            path = item.split('/')
            pyname = path[-1]
            if pyname.endswith('.py'):
                names.append(pyname.replace('.py',''))
        self.load_modules(*names,cwd=cwd)
    def load_modules(self,*names,**kw):
        cwd = None
        if kw:
            if kw.has_key('cwd'):
                cwd = kw['cwd']
        for name in names: self.load_module(name,cwd=cwd)
    def load_module(self,name,cwd=None):
        klass = {}
        try:
            if cwd:
                m = imp.load_module(name,*imp.find_module(name,[cwd]))
            else:
                m  = imp.load_module(name,*imp.find_module(name))
            self.update_by_module(m)
        except SyntaxError: print 'Syntax Error', name
        except ImportError: print 'Import Error', name
        except AttributeError: print 'Attribute Error', name
    def made_decision(self):
        tv = self.children['klass']
        id = tv.selection()
        if len(id):
            self.selection = self.klass[id[0]]
        self.destroy()
    def update(self,modules=None):
        if not modules:
            modules = globals()
        for key in modules:
            self.update_by_module(modules[key])
    def update_by_module(self,module):
        if inspect.isclass(module):
            if self.iscomponent(module):
                self.add_item(module)
        if inspect.ismodule(module):
            components = self.extract_components(module)
            for component in components:
                self.add_item(component)
    def extract_components(self,module):
        components = []
        items = inspect.getmembers(module,lambda x:inspect.isclass(x))
        for key,cls in items:
            if self.iscomponent(cls):
                components.append(cls)
        return components
    def iscomponent(self,klass):
        family = inspect.getmro(klass)
        for c in family:
            name = repr(c).split()[1]
            if name == 'pylaf.core.Component' or name == 'core.Component':
                return True
        return False
    def add_item(self,cls):
        for id in self.klass:
            if self.klass[id] == cls:
                return
        tv = self.children['klass']
        path = repr(cls).split()[1].split('.')
        parent = ''
        for child in path[:-1]:
            flag = False
            for id in tv.get_children(parent):
                if child == tv.item(id)['text']:
                    flag = True
            if flag:
                parent = id
            else:
                parent = tv.insert(parent,'end',text=child)
        id = tv.insert(parent,'end',text=path[-1])
        self.klass[id] = cls
        
class ComponentTree(ttk.Treeview):
    class Menu(Tkinter.Menu):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Menu.__init__(self,master,cnf,**kw)
            self.add_command(label='New Sibling',command=core.CallWrapper(master.newsibling))
            self.add_command(label='New Child',command=core.CallWrapper(master.newchild))
            self.add_command(label='Delete',command=core.CallWrapper(master.mydelete))
            self.add_separator()
            self.add_command(label='Set Export',command=core.CallWrapper(master.set_export))
            self.add_command(label='Reset Export',command=core.CallWrapper(master.reset_export))
    def __init__(self,master,**kw):
        ttk.Treeview.__init__(self,master,column=2,**kw)
        self.queue = []
        # ポップアップメニューの設定
        self.Menu(self,name='menu')
        self.bind(BUTTON_R, lambda e:self.children['menu'].tk_popup(e.x_root,e.y_root))
        #
        self.components = weakref.WeakValueDictionary()
        self.components[''] = core.Root()
        self.populate('',core.Root())
        #
        self.tag_configure('export',foreground='#0000FF')
        #
        self.loop()
    def set_export(self):
        id = self.myfocus()
        if not (id == '' or id == None):
            self.item(id,tag='export')
    def reset_export(self):
        id = self.myfocus()
        if not (id == '' or id == None):
            if self.item(id)['tags'].count('export'):
                tags = self.item(id)['tags']
                tags.remove('export')
                self.item(id,tags=tags)
    def loop(self):
        self.refresh()
        self.after(500,self.loop)
    def refresh(self):
        self.remove_item()
        self.populate('',core.Root())
        self.markup()
    def markup(self):
        for obj in self.queue:
            id = None
            for item in self.components:
                if obj == self.components[item]:
                    id = item
            if id:
                self.item(id,tag='export')
        self.queue = []
    def remove_item(self,item=None):
        # 幅方向優先削除
        # 削除候補の検索
        remove = []
        for key in self.get_children(item):
            if not self.components.has_key(key):
                remove.append(key)
        # 削除作業
        for key in remove:
            self.delete(key)
        # 深さ方向の処理
        for child in self.get_children(item):
            self.remove_item(child)
    def myfocus(self):
        item = self.selection()
        if    len(item) == 0: return ''
        elif  len(item) == 1: return item[0]
        else: return
    #def update(self):
    #    self.delete(*self.get_children())
    #    self.populate('',core.Root())
    def populate(self,node,component):
        parent = str(node)
        for key in component.children:
            c = component.children[key]
            children = self.get_children(parent)
            flag = False
            for child in children: # もし追加したいコンポーネントがあってもchildrenの中にすでに存在すれば追加しないでparentだけ更新する
                if c == self.components[child]:
                    flag = True
                    id = child
            if not flag:
                id = self.insert(parent,'end',text=classname(c),values=[key])
                self.components[id] = c
            self.populate(id,c)
    def newsibling(self):
        key = self.myfocus()
        if key == None: return
        key    = self.parent(key)
        parent = self.components[key]
        cc = ComponentChooser(Tkinter.Toplevel(self))
        cc.master.destroy()
        if cc.selection:
            item = cc.selection(parent)
            self.queue.append(item)
            self.master.newtoplevel(item)
    def newchild(self):
        key = self.myfocus()
        if key == None: return
        parent = self.components[key]
        cc = ComponentChooser(Tkinter.Toplevel(self))
        cc.master.destroy()
        if cc.selection:
            item = cc.selection(parent)
            self.queue.append(item)
            self.master.newtoplevel(item)
    def mydelete(self):
        key = self.myfocus()
        if key == None: return
        component = self.components[key]
        component.destroy()
        self.delete(key)

class WidgetTree(ttk.Treeview):
    class Menu(Tkinter.Menu):
        def __init__(self,master=None,cnf={},**kw):
            Tkinter.Menu.__init__(self,master,cnf,**kw)
            self.add_command(label='New Toplevel as Console',command=core.CallWrapper(master.newtoplevel))
            self.add_command(label='New Toplevel as Console',command=core.CallWrapper(master.newidptoplevel))
            self.add_command(label='New Sibling as Embed',command=core.CallWrapper(master.newsibling))
            self.add_command(label='New Child as Embed',command=core.CallWrapper(master.newchild))
            self.add_command(label='Delete',command=core.CallWrapper(master.mydelete))
            self.add_separator()
            self.add_command(label='Refresh',command=core.CallWrapper(master.refresh))
    def __init__(self,master,**kw):
        ttk.Treeview.__init__(self,master,column=2,**kw)
        tk = self
        while tk.master: tk = tk.master
        self.widgets = weakref.WeakValueDictionary()
        self.widgets[''] = tk
        self.populate('',tk)
        # ポップアップメニューの設定
        self.Menu(self,name='menu')
        self.bind(BUTTON_R, lambda e:self.children['menu'].tk_popup(e.x_root,e.y_root))
        self.loop()
    def loop(self):
        self.refresh()
        self.after(500,self.loop)
    def populate(self,node,widget):
        parent = str(node)
        for key in widget.children:
            # Kitchenを表示しない
            #flag = False
            #for mykey in component.children[key].children:
            #    if classname(component.children[key].children[mykey]) == 'Kitchen':
            #        flag = True
            #        break
            #if flag: return
            #
            c = widget.children[key]
            children = self.get_children(parent)
            flag = False
            for child in children: # もし追加したいコンポーネントがあってもchildrenの中にすでに存在すれば追加しないでparentだけ更新する
                if c == self.widgets[child]:
                    flag = True
                    id = child
            if not flag:
                id = self.insert(parent,'end',text=classname(c),values=[key])
                self.widgets[id] = c
            self.populate(id,c)
    def refresh(self):
        tk = self
        while tk.master: tk = tk.master
        self.remove_item()
        self.populate('',tk)
    def remove_item(self,item=None):
        # 幅方向優先削除
        # 削除候補の検索
        remove = []
        for key in self.get_children(item):
            if not self.widgets.has_key(key):
                remove.append(key)
        # 削除作業
        for key in remove:
            self.delete(key)
        # 深さ方向の処理
        for child in self.get_children(item):
            self.remove_item(child)
    def myfocus(self):
        item = self.selection()
        if    len(item) == 0: return ''
        elif  len(item) == 1: return item[0]
        else: return
    def newtoplevel(self,component=None):
        if component == None:
            component = self.master.focus()
#        component = self.master.focus()
        if component == None: return
        key    = ''
        parent = self.widgets[key]
        top    = Tkinter.Toplevel(parent)
        gui.Console(top,component)
        item   = self.insert(key,'end',text=classname(top),values=[repr(id(top))])
        self.widgets[item] = top
        self.populate(item,top)
    def newidptoplevel(self):
        component = self.master.focus()
        if component == None: return
        key    = ''
        parent = self.widgets[key]
        top    = Tkinter.Toplevel(parent)
        gui.Console(top,component,sync=False)
        item   = self.insert(key,'end',text=classname(top),values=[repr(id(top))])
        self.widgets[item] = top
        self.populate(item,top)
    def newsibling(self): pass
    def newchild(self): pass
    def mydelete(self): pass

class CanvasItem:
    STATE_NORMAL, STATE_HIGHLIGHT = range(2)
    def __init__(self,canvas,cid,x,y,**kw):
        self.tag     = tag = str(uuid.uuid4())
        self.canvas  = weakref.ref(canvas)
        self.cid     = cid
        self.x       = x
        self.y       = y
        self.state   = CanvasItem.STATE_NORMAL
        self.outline = canvas.create_rectangle(x-50,y-20,x+50,y+20,width=2,fill='#FFFFF0',tag=tag)
        self.label   = canvas.create_text(x,y,text=self.format(cid),tag=tag)
        canvas.tag_bind(tag,'<Button-1>', self.buttonpress1)
        canvas.tag_bind(tag,'<ButtonRelease-1>', self.buttonrelease1)
        canvas.tag_bind(tag,'<Button1-Motion>', self.buttonmotion1)
    def locate(self):
        x0, y0, x1, y1 = self.canvas().bbox(self.outline)
        return (x0 + x1) / 2., (y0 + y1) / 2.
    def iscontain(self,item):
        return self.outline == item or self.label == item
    def highlight(self):
        if self.state == CanvasItem.STATE_HIGHLIGHT: return
        self.state = CanvasItem.STATE_HIGHLIGHT
        self.canvas().itemconfig(self.outline,width=3)
    def lift(self):
        self.canvas().lift(self.outline)
        self.canvas().lift(self.label)
    def normal(self):
        if self.state == CanvasItem.STATE_NORMAL: return
        self.state = CanvasItem.STATE_NORMAL
        self.canvas().itemconfig(self.outline,width=2)
    def mode_wire(self):
        self.canvas().mode_wire(self)
        self.highlight()
    def buttonpress1(self,e):
        self.x0, self.y0 = e.x, e.y
        self.aid = self.canvas().after(1500,self.mode_wire)
    def buttonrelease1(self,e):
        if   self.canvas().is_wire():
            self.canvas().link()
            self.canvas().mode_move()
        elif self.canvas().is_move():
            self.canvas().after_cancel(self.aid)
    def buttonmotion1(self,e):
        self.lift()
        self.canvas().after_cancel(self.aid)
        if   self.canvas().is_move(): self.move(e)
        elif self.canvas().is_wire(): self.canvas().dragging_wire(e)
    def move(self,e):
        canvas = self.canvas()
        x1, y1 = e.x, e.y
        dx, dy = x1 - self.x0, y1 - self.y0
        canvas.move(self.outline, dx, dy)
        canvas.move(self.label,   dx, dy)
        self.x, self.y = self.x + dx, self.y + dy
        self.x0, self.y0 = x1, y1
        canvas.move_wire(self,dx,dy)
    def format(self,cid):
        if cid == '':
            cname  = 'Root'
            name   = ''
        else:
            canvas = self.canvas()
            cname  = classname(canvas.components()[cid])
            name   = canvas.components()[cid]._name
        return '%s\n%s' % (name,cname)
    def delete(self):
        self.canvas().delete(self.tag)

class CanvasLine:
    def __init__(self,canvas,terminals):
        self.terminals = terminals
        self.canvas    = weakref.ref(canvas)
        self.bx, self.by = terminals[0].locate()
        self.ex, self.ey = terminals[1].locate()
        self.draw()
        canvas.tag_bind(self.lid,'<ButtonPress-1>', self.buttonpress1)
        canvas.tag_bind(self.lid,'<Enter>', self.enter)
        canvas.tag_bind(self.lid,'<Leave>', self.leave)
    def buttonpress1(self,e):
        components = self.canvas().components()
        terminals = (components[self.terminals[0].cid],components[self.terminals[1].cid])
        self.canvas().popup_linker(terminals)
    def enter(self,e):
        components = self.canvas().components()
        self.canvas().itemconfig(self.lid,width=2)
        terminals = (components[self.terminals[0].cid],components[self.terminals[1].cid])
        self.canvas().show_links(terminals)
    def leave(self,e):
        self.canvas().itemconfig(self.lid,width=1)
        self.canvas().hide_links()
    def eq(self,terminals):
        t1, t2 = terminals
        myt1, myt2 = self.terminals
        return (t1 == myt1 and t2 == myt2) or (t1 == myt2 and t2 == myt1)
    def move(self):
        bx, by = self.terminals[0].locate()
        ex, ey = self.terminals[1].locate()
        if self.bx == bx and self.by == by and self.ex == ex and self.ey == ey: return
        self.bx, self.by, self.ex, self.ey = bx, by, ex, ey
        self.delete()
        self.draw()
    def draw(self):
        self.lid = self.canvas().create_line(self.bx, self.by, self.ex, self.ey)
        self.canvas().lower(self.lid)
    def delete(self):
        self.canvas().delete(self.lid)
    
class ComponentLabel(Tkinter.Label):
    def __init__(self,master,component=None,cnf={},**kw):
        Tkinter.Label.__init__(self,master,cnf,**kw)
        if component:
            self.insert(component)
        else:
            self.delete()
    def delete(self):
        self.configure(text=' \n ')
    def insert(self,component):
        self.configure(text='%s\n%s' % (component._name,classname(component)))

class LinkViewer(Tkinter.Listbox):
    def __init__(self,master,withlabel=False,cnf={},**kw):
        frame = Tkinter.Frame(master)
        self.withlabel = withlabel
        if withlabel:
            lentry = ComponentLabel(frame)
            rentry = ComponentLabel(frame)
            self.lentry = weakref.ref(lentry)
            self.rentry = weakref.ref(rentry)
        Tkinter.Listbox.__init__(self,frame,cnf,**kw)
        if kw:
            if kw.has_key('name'):
                del kw['name']
        right = Tkinter.Listbox(frame,cnf,**kw)
        #
        self.frame = weakref.ref(frame)
        self.right = weakref.ref(right)
    def activate(self,terminals):
        pairs = self.pairs(terminals)
        lpairs = [pair[0] for pair in pairs]
        rpairs = [pair[1] for pair in pairs]
        self.delete(0,Tkinter.END)
        self.insert(Tkinter.END,*lpairs)
        self.right().delete(0,Tkinter.END)
        self.right().insert(Tkinter.END,*rpairs)
        if self.withlabel:
            self.lentry().insert(terminals[0])
            self.rentry().insert(terminals[1])
    def deactivate(self):
        self.delete(0,Tkinter.END)
        self.right().delete(0,Tkinter.END)
        if self.withlabel:
            self.lentry().delete()
            self.rentry().delete()
    def pairs(self,terminals):
        pairs = []
        # terminalに含まれるストレージのリストを生成
        storages = {}
        for terminal in terminals:
            m = inspect.getmembers(terminal,lambda x:iscontain(x,core.Port))
            for name, port in m:
                if not storages.has_key(port.subject):
                    storages[port.subject] = [(terminal,name)]
                else:
                    storages[port.subject].append((terminal,name))
        # 監視ポートがただひとつならばラインデータベースから削除
        removable = []
        for sid in storages:
            portlist = storages[sid]
            if len(portlist) == 1:
                removable.append(sid)
        for sid in removable:
            del storages[sid]
        # pairを生成する
        for sid in storages:
            links = self.pairing(terminals,storages[sid])
            for item in links:
                pairs.append(item)
        return pairs
    def pairing(self,terminals,ports):
        def select(component,ports):
            result = []
            for c, name in ports:
                if c == component:
                    result.append(name)
            return result
        portsa = select(terminals[0],ports)
        portsb = select(terminals[1],ports)
        links = []
        for namea in portsa:
            for nameb in portsb:
                links.append((namea,nameb))
        return links
    def grid(self,**kw):
        self.frame().grid(**kw)
        self.layout()
    def pack(self,**kw):
        self.frame().pack(**kw)
        self.layout()
    def layout(self):
        if self.withlabel:
            self.lentry().grid(row=0,column=0)
            self.rentry().grid(row=0,column=1)
        Tkinter.Listbox.grid(self,row=1,column=0)
        self.right().grid(row=1,column=1)
    def lselection(self):
        selection = self.curselection()
        if not len(selection): return
        return self.get(0,Tkinter.END)[int(selection[0])]
    def rselection(self):
        selection = self.right().curselection()
        if not len(selection): return
        return self.right().get(0,Tkinter.END)[int(selection[0])]

class Linker(Tkinter.Frame):
    class PortPicker(Tkinter.Listbox):
        def __init__(self,master,entry,component,cnf={},**kw):
            Tkinter.Listbox.__init__(self,master,cnf,**kw)
            m = inspect.getmembers(component,lambda x:iscontain(x,core.Port))
            entry.delete(0,Tkinter.END)
            self.insert(Tkinter.END, *[name for name, port in m])
            self.bind('<ButtonRelease-1>',self.selection)
            self.entry = weakref.ref(entry)
        def selection(self,e):
            curselection = self.curselection()
            if not len(curselection): return
            name = Tkinter.Listbox.get(self,0,Tkinter.END)[int(curselection[0])]
            entry = self.entry()
            entry.delete(0,Tkinter.END)
            entry.insert(0,name)
    def __init__(self,master,terminals,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.master.title('Linker: PyLAF Kitchen')
        a, b = terminals
        ComponentLabel(self,a).grid(row=0,column=0)
        pickera = Tkinter.Entry(self); pickera.grid(row=1,column=0)
        Linker.PortPicker(self,pickera,a).grid(row=2,column=0)
        cframe = Tkinter.Frame(self,name='cframe'); cframe.grid(row=1,column=1)
        Tkinter.Button(cframe,text='unlink',command=self.unlinka).grid(row=0,column=0)
        Tkinter.Button(cframe,text='<',command=self.alinkb).grid(row=0,column=1)
        Tkinter.Button(cframe,text='>',command=self.blinka).grid(row=0,column=2)
        Tkinter.Button(cframe,text='unlink',command=self.unlinkb).grid(row=0,column=3)
        linkviewer = LinkViewer(self); linkviewer.grid(row=2,column=1)
        ComponentLabel(self,b).grid(row=0,column=2)
        pickerb = Tkinter.Entry(self); pickerb.grid(row=1,column=2)
        Linker.PortPicker(self,pickerb,b).grid(row=2,column=2)
        #
        linkviewer.activate(terminals)
        #
        self.a = weakref.ref(a)
        self.b = weakref.ref(b)
        self.pickera = weakref.ref(pickera)
        self.pickerb = weakref.ref(pickerb)
        self.linkviewer = weakref.ref(linkviewer)
    def alinkb(self):
        pa = self.pickera().get()
        pb = self.pickerb().get()
        try:
            getattr(self.a(),pa).link(getattr(self.b(),pb))
            self.master.destroy()
        except AttributeError: pass
    def blinka(self):
        pa = self.pickera().get()
        pb = self.pickerb().get()
        try:
            getattr(self.b(),pb).link(getattr(self.a(),pa))
            self.master.destroy()
        except AttributeError: pass
    def unlinka(self):
        name = self.linkviewer().lselection()
        if name:
            getattr(self.a(),name).unlink()
        self.linkviewer().deactivate()
        self.linkviewer().activate((self.a(),self.b()))
    def unlinkb(self):
        name = self.linkviewer().rselection()
        if name:
            getattr(self.b(),name).unlink()
        self.linkviewer().deactivate()
        self.linkviewer().activate((self.a(),self.b()))
    def components(self):
        return self.master.master.components()

class CollaborationMap(Tkinter.Canvas):
    MODE_MOVE, MODE_WIRE = range(2)
    def __init__(self,master,cnf={},**kw):
        Tkinter.Canvas.__init__(self,master,cnf,**kw)
        self.items = weakref.WeakValueDictionary()
        self.lines = []
        self.mode  = CollaborationMap.MODE_MOVE
        self.bitem = None
        self.eitem = None
        self.loop()
    def loop(self):
        self.refresh()
        self.after(500,self.loop)
    def components(self): return self.master.components()
    def show_links(self,terminals): self.master.show_links(terminals)
    def hide_links(self): self.master.hide_links()
    def popup_linker(self,terminals):
        Linker(Tkinter.Toplevel(self),terminals).pack()
    def mode_wire(self,sender):
        self.mode = CollaborationMap.MODE_WIRE
        for cid in self.items:
            if self.items[cid] == sender:
                self.bitem = cid
    def mode_move(self):
        self.mode = CollaborationMap.MODE_MOVE
        for cid in self.items:
            self.items[cid].normal()
        self.bitem, self.eitem = None, None
    def is_wire(self): return self.mode == CollaborationMap.MODE_WIRE
    def is_move(self): return self.mode == CollaborationMap.MODE_MOVE
    def dragging_wire(self,e):
        id = self.find_closest(e.x,e.y)[0]
        for cid in self.items:
            item = self.items[cid]
            if item.iscontain(id):
                if cid == self.bitem:
                    self.eitem = None
                    continue
                else:
                    self.eitem = cid
                    item.highlight()
            else:
                if not cid == self.bitem:
                    item.normal()
    def link(self):
        if self.bitem == None or self.eitem == None: return
        components = self.components()
        b, e = components[self.bitem], components[self.eitem]
        #b.entry4.link(e.entry4)
        Linker(Tkinter.Toplevel(self),(b,e)).pack()
    def move_wire(self,item,dx,dy):
        for line in self.lines:
            line.move()
    def refresh(self):
        components = self.components()
        # remove obsolete items
        items = self.items.keys()
        for cid in components:
            if items.count(cid):
                items.remove(cid)
        for cid in items:
            self.items[cid].delete()
        # append new items
        items = []
        for cid in components:
            if not self.items.has_key(cid):
                item = CanvasItem(self,cid,200*random.random()+100,200*random.random()+50)
                self.items[cid] = item
        #
        #
        #
        # ストレージの全リストとそれを監視しているポートを２つ以上保持しているコンポーネントとポート名のデータベースの生成
        storages = {} # コンポーネントやストレージへの参照を含むのでメンバとして保持しちゃだめ
        for cid in components:
            c = components[cid]
            m = inspect.getmembers(c,lambda x:iscontain(x,core.Port))
            for name, port in m:
                if not storages.has_key(port.subject):
                    storages[port.subject] = [(c,name)]
                else:
                    storages[port.subject].append((c,name))
        #
        removable = []
        for sid in storages:
            portlist = storages[sid]
            # 監視ポートがただひとつならばラインデータベースから削除
            if len(portlist) == 1:
                removable.append(sid)
        for sid in removable:
            del storages[sid]
        #
        removable = []
        for sid in storages:
            portlist = storages[sid]
            # 単一の監視コンポーネントの複数のポートから参照されている場合も削除
            # !!! 現状ではラインの表現方法を決めてないが、将来はこれにも対応すること !!!
            component = portlist[0]
            if portlist.count(component) == len(portlist):
                removable.append(sid)
        for sid in removable:
            del storages[sid]
        # 関連したコンポーネント対のリストを生成（順不同は同一とみなす）
        def pairs(o,clist):
            tmpclist = list(clist)
            last     = tmpclist.pop(0) # 先頭の要素を取り出し
            for c in tmpclist:
                flag = False
                for item in o:
                    if (item[0] == c and item[1] == last) or\
                       (item[0] == last and item[1] == c):
                        flag = True
                if not flag:
                    o.append((last,c))
            if len(tmpclist):
                pairs(o,tmpclist)
        links = []
        for sid in storages:
            plist = storages[sid]
            clist = [p[0] for p in plist]
            pairs(links,clist)
        cids = {}
        for cid in components:
            cids[components[cid]] = cid
        linksbycid = []
        for pair in links:
            linksbycid.append((cids[pair[0]],cids[pair[1]]))
        # ラインの削除
        removable = []
        for line in self.lines:
            flag = False
            for pair in linksbycid:
                if line.eq(pair):
                    flag = True
            if not flag:
                removable.append(line)
        for line in removable:
            self.lines.remove(line)
            line.delete()
        # ラインの描画
        for pair in linksbycid:
            pair = (self.items[pair[0]],self.items[pair[1]])
            flag = False
            for line in self.lines: # すでに描画してあるペアか検査して
                if line.eq(pair):
                    flag = True
            if not flag: # まだ描画していなければ描画する
                self.lines.append(CanvasLine(self,pair))

class Kitchen(Tkinter.Frame):
    def __init__(self,master,cnf={},**kw):
        Tkinter.Frame.__init__(self,master,cnf,**kw)
        self.master.title('PyLAF Kitchen')
        tab = ttk.Notebook(self,name='tabpane'); tab.grid(row=1,column=0)
        tab.add(ComponentTree(self,name='componenttree'),text='Components')
        tab.add(WidgetTree(self,name='widgettree'),text='Widgets')
        CollaborationMap(self,name='collaborationmap',width=640,height=400).grid(row=0,column=0,columnspan=2)
        linkviewer = LinkViewer(self,withlabel=True); linkviewer.grid(row=1,column=1)
        self.linkviewer = weakref.ref(linkviewer)
    def focus(self):
        item = self.children['componenttree'].myfocus()
        if item == None or item == '': return
        return self.children['componenttree'].components[item]
    def components(self):
        return self.children['componenttree'].components
    def show_links(self,terminals):
        self.linkviewer().activate(terminals)
    def hide_links(self):
        self.linkviewer().deactivate()
    def newtoplevel(self,component=None):
        self.children['widgettree'].newtoplevel(component)
        
if __name__ == '__main__':
    import gui
    class MyComponent(core.Component):
        class Control(gui.Panel):
            def __init__(self,master=None,cnf={},**kw):
                gui.Panel.__init__(self,master,cnf,**kw)
                builder = gui.TableBuilder(self)
                builder.add(Tkinter.Label,text='Entry1').grid()
                builder.add(widgets.Entry,name='entry1').grid()
                #self.append(gui.Entry(self,name='entry1'),label='Entry1')
        #class Menu(gui.MenuFactory):
        #    DESC = [['PyLAF', ['Kitchen', 'command']],]
        #    def __init__(self,master=None,cnf={},**kw):
        #        gui.MenuFactory.__init__(self,master,self.DESC,cnf,**kw)
        #    def assign(self,comp):
        #        self.children['pylaf'].assign('kitchen',self.invoke_kitchen)
        #    def invoke_kitchen(self):
        #        tk = self
        #        while tk.master: tk = tk.master
        #        Kitchen(Tkinter.Toplevel(tk)).pack()
        def __init__(self,master=None,name=None):
            core.Component.__init__(self,master,name)
            self.entry1 = core.Port(0.)
            self.entry2 = core.Port(0.)
            self.entry3 = core.Port(0.)
            self.entry4 = core.Port(0.)
    
    tk = Tkinter.Tk()
    basic = gui.Console(tk,MyComponent(core.Root()))
    basic.master.title('basic sample')
    c = basic.comp()
    my   = MyComponent(core.Root(),name='component')
    my1  = MyComponent(c,name='component1')
    my2  = MyComponent(c,name='component2')
    my11 = MyComponent(c.children['component1'],name='component11')
    my.entry1.link(my1.entry1)
    my.entry2.link(my1.entry2)
    my.entry1.link(my2.entry1)
    del my, my1, my2, my11
    Tkinter.mainloop()
    